
#include "zeus.h"

int civ_to_num( name )
char *name;
{
	int civ;
	for( civ = 0; civ < NumCivs; civ++ )
		if( !strcasecmp( Civ[ civ ].abbr, name ) )
			return( civ );
	for( civ = 0; civ < NumCivs; civ++ )
		if( !strcasecmp( Civ[ civ ].name, name ) )
			return( civ );
	return( -1 );
}

get_civs( filename )
char *filename;
{
	FILE *fileptr;
	char line[ 128 ];
	char name[ 32 ], abbr[ 8 ], player[ 32 ];
	int active;

	if( !( fileptr = fopen( filename, "r" ) ) )
		fopen_error( filename );

	while( fgetline( fileptr, line, 128 ) != -1 )
	{
		sscanf( line, "%s %s %d %s", name, abbr, &active, player );

		Civ[ NumCivs ].name = (char *)malloc( strlen( name ) + 1 );
		strcpy( Civ[ NumCivs ].name, name );

		Civ[ NumCivs ].abbr = (char *)malloc( strlen( abbr ) + 1 );
		strcpy( Civ[ NumCivs ].abbr, abbr );

		Civ[ NumCivs ].active = active;

		Civ[ NumCivs ].player = (char *)malloc( strlen( player ) + 1 );
		strcpy( Civ[ NumCivs ].player, player );

		if( active == 1 )
			NumPlayers++;

		NumCivs++;
	}

	fclose( fileptr );
}

get_census_order()
{
	char corder[ 16 ][ 6 ];
	int index, civ, nelem = 0;

	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			if( Civ[ civ ].mil )
				sprintf( &corder[ nelem ][ 0 ], "%03d%02d",
					2 * MaxTokens - Civ[ civ ].census, civ );
			else
				sprintf( &corder[ nelem ][ 0 ], "%03d%02d",
					MaxTokens - Civ[ civ ].census, civ );
			nelem++;
		}
	}
	qsort( &corder[ 0 ], nelem, 6, (void *)strcmp );
	for( index = 0; index < nelem; index++ )
	{
		sscanf( &corder[ index ][ 3 ], "%2d", &civ );
		CensusOrder[ index ] = civ;
	}
}

get_standings()
{
	char corder[ 16 ][ 7 ];
	int index, civ, nelem = 0;

	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			Civ[ civ ].score = total_civcard_points( civ ) +
				( Civ[ civ ].cities * 50 ) + ( Civ[ civ ].astpos * 100 );
			sprintf( &corder[ nelem ][ 0 ], "%04d%02d", Civ[ civ ].score, civ );
			nelem++;
		}
	}
	qsort( &corder[ 0 ], nelem, 7, (void *)strcmp );
	for( index = 0; index < nelem; index++ )
	{
		sscanf( &corder[ index ][ 4 ], "%2d", &civ );
		Standings[ nelem - index - 1 ] = civ;
	}
}

print_standings( output )
FILE *output;
{
	int index, citypts, advpts, astpts, totalpts;

	fprintf( output, "\nStandings\n" );
	fprintf( output, "=========\n\n" );
	fprintf( output, "Civilization  Player   Age           Cities   Advances    AST   Total\n" );
	fprintf( output, "------------  -------  ------------- ------   --------   ----   -----\n" );
	for( index = 0; index < NumCivs; index++ )
	{
		if( Civ[ index ].active == 1 )
		{
			citypts = Civ[ index ].cities * 50;
			advpts = total_civcard_points( index );
			astpts = ( Civ[ index ].astpos + Civ[ index ].astnew ) * 100;
			totalpts = citypts + advpts + astpts;
			fprintf( output, "%-12s  %-7s  %-12s",
				Civ[ index ].name, Civ[ index ].player, civage( index ) );
			if( next_age( index ) )
				fprintf( output, "*" );
			else
				fprintf( output, " " );
			fprintf( output, " %6d   %8d   %4d",
				citypts, advpts, astpts );
			if( Civ[ index ].astnew == 1 )
				fprintf( output, "+" );
			else if( Civ[ index ].astnew == -1 )
				fprintf( output, "-" );
			else
				fprintf( output, " " );
			fprintf( output, "  %5d\n", totalpts );
		}
	}
	fprintf( output, "\n\"*\" = civilization about to enter next age\n" );
}

print_general_info( output, turn )
FILE *output;
int turn;
{
	int civ, index;

	fprintf( output, "\nGeneral Info\n" );
	fprintf( output, "==============\n\n" );
	fprintf( output, "Civilization   Census  Cities  Ships  Stock  Tax  Treasury  Cards  AST\n" );
	fprintf( output, "------------   ------  ------  -----  -----  ---  --------  -----  ---\n" );
	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			fprintf( output, "%-12s  %7d %7d %6d %6d %4d %9d %6d %4d\n",
				Civ[ civ ].name,
				Civ[ civ ].census,
				Civ[ civ ].cities,
				Civ[ civ ].ships,
				Civ[ civ ].stock,
				Civ[ civ ].taxrate,
				Civ[ civ ].treasury,
				Civ[ civ ].ncards,
				Civ[ civ ].astpos + Civ[ civ ].astnew );
		}
	}
	get_census_order();
	fprintf( output, "\n" );
	fprintf( output, "Order of Movement (Turn %d)\n", turn );
	fprintf( output, "=============================\n" );
	for( civ = 0; civ < NumPlayers; civ++ )
	{
		fprintf( output, "%-12s", Civ[ CensusOrder[ civ ] ].name );
		if( Civ[ CensusOrder[ civ ] ].mil )
			fprintf( output, " military\n" );
		else
			fprintf( output, "\n" );
	}

	if( turn == Turn )
		return;

	fprintf( output, "\n" );
	fprintf( output, "Calamities\n" );
	fprintf( output, "==========\n" );
	if( !CalamityOccured )
		fprintf( output, "None\n" );
	for( index = 0; index < NTCards; index++ )
	{
		if( Card[ index ].cal )
		{
			for( civ = 0; civ < NumCivs; civ++ )
			{
				if( Civ[ civ ].cards[ index ] )
				{
					fprintf( output, "%-20s - %s\n",
						Card[ index ].name, Civ[ civ ].name );
					/*
					Civ[ civ ].cards[ index ] = 0;
					Civ[ civ ].ncards--;
					*/
				}
			}
		}
	}
}

print_card_purchases( output )
FILE *output;
{
	int civ, count, number, value, firstvalue;
	int totalvalue, totalcredits, totalcost;
	int plus;

	fprintf( output, "Card Purchases & Expenditures\n" );
	fprintf( output, "=============================\n" );
	fprintf( output, "\n" );
	fprintf( output, "Purchases\n" );
	fprintf( output, "---------\n" );
	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == INPLAY )
		{
			fprintf( output, "%-12s\n", Civ[ civ ].name );
			totalvalue = 0;
			totalcredits = 0;
			totalcost = 0;
			for( count = 0; count < NUM_TECHS; count++ )
			{
				if( Civ[ civ ].newtech[ count ] )
				{
					totalvalue += Tech[ count ].value;
					totalcredits += Civ[ civ ].techcredits[ count ];
					totalcost += Civ[ civ ].techcost[ count ];
					fprintf( output, "\t%-13s %3d - %3d = %3d ",
						Tech[ count ].name,
						Tech[ count ].value,
						Civ[ civ ].techcredits[ count ],
						Civ[ civ ].techcost[ count ] );
					for( value = 5; value < 30; value += 5 )
					{
						firstvalue = 1;
						for( number = 0; number < NUM_TECHS; number++ )
							if( Civ[ civ ].tech[ number ] &&
								!Civ[ civ ].newtech[ number ] &&
								( Credits[ number ][ count ] == value ) )
							{
								if( firstvalue )
								{
									fprintf( output, "%2d:", value );
									firstvalue = 0;
								}
								fprintf( output, "%3s ", Tech[ number ].abbr );
							}
					}
					fprintf( output, "\n" );
				}
			}
			if( totalvalue > 0 )
			{
				fprintf( output, "\t              %3d - %3d = %3d\n",
					totalvalue, totalcredits, totalcost );
			}
			else
			{
				fprintf( output, "\tNone\n" );
			}
			fprintf( output, "\n" );
		}
	}
	fprintf( output, "Expenditures\n" );
	fprintf( output, "------------\n" );
	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			fprintf( output, "%-12s %3d:",
				Civ[ civ ].name, Civ[ civ ].money );
			plus = 0;
			for( count = 0; count < NTCards; count++ )
			{
				if( Civ[ civ ].discards[ count ] && !Card[ count ].cal )
				{
					if( plus )
						fprintf( output, " +" );
					plus++;
					fprintf( output, " %d-%s(%d)",
						Civ[ civ ].discards[ count ],
						Card[ count ].abbr,
						Civ[ civ ].discard_value[ count ] );
					if( Civ[ civ ].cardmined == count )
						fprintf( output, "*" );
				}
			}
			if( Civ[ civ ].change )
			{
				if( plus )
					fprintf( output, " +" );
				fprintf( output, " %dT(%d)",
					Civ[ civ ].change,
					Civ[ civ ].change );
			}
			fprintf( output, "\n" );
		}
	}
	fprintf( output, "\n\"*\" - set's value was increased by Mining\n" );
}

list_status()
{
	char command[ 128 ];
	int index;
	int citypts, advpts, astpts, totalpts;
	time_t tloc;
	int phase;

	time( &tloc );
	day = ( tloc - starttime ) / 86400;

	get_mapboard();

	phase = 2 * Phase + PhaseComplete;
	if( phase >= 15 )
		handle_growth();
	else if( phase >= 13 )
		handle_calamities();
	/*
	else if( phase >= 11 )
	*/
	else if( phase >= 10 )
		handle_deal();
	else if( phase >= 9 )
		handle_cities();
	else if( phase >= 7 )
		handle_conflict();
	else if( phase >= 4 )
		handle_move();
	else if( phase >= 3 )
		handle_ships();

	tempfile = fopen( TEMPFILE, "w+" );
	fprintf( tempfile, "Civilization Game Status\n" );
	fprintf( tempfile, "========================\n\n" );
	fprintf( tempfile, "Turn:   %d\n", Turn );
	/*
	if( Years[ Turn ] < 0 )
		fprintf( tempfile, " : %d B.C.\n", -Years[ Turn ] );
	else
		fprintf( tempfile, " : %d A.D.\n", Years[ Turn ] );
	*/
	fprintf( tempfile, "Phase:  %s\n", PhaseName[ Phase ] );
	fprintf( tempfile, "Status: %s\n", PhaseStatus[ PhaseComplete ] );
	/*
	fprintf( tempfile, "Day:    %d\n", day );
	*/

	if( phase <= 4 )
	{
		int nextmove = -1;
		fprintf( tempfile, "\nMove order: " );
		for( index = 0; index < NumPlayers; index++ )
		{
			fprintf( tempfile, "%s ", Civ[ CensusOrder[ index ] ].name );
			if( !Civ[ CensusOrder[ index ] ].moved && ( nextmove == -1 ) )
				nextmove = CensusOrder[ index ];
		}
		fprintf( tempfile, "\n" );
		fprintf( tempfile, "Next move:  " );
		fprintf( tempfile, "%s\n", Civ[ nextmove ].name );
	}

	print_standings( tempfile );

	print_general_info( tempfile, Turn );

	fprintf( tempfile, "\n" );
	fclose( tempfile );

	if( phase >= 3 )
	{
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "Ships\n" );
		fprintf( tempfile, "=====\n\n" );
		fclose( tempfile );
		sprintf( command, "cat %s >> %s", SHIPSFILE, TEMPFILE );
		system( command );
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "\n" );
		fclose( tempfile );
	}

	if( phase >= 4 )
	{
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "Movement\n" );
		fprintf( tempfile, "========\n\n" );
		fclose( tempfile );
		sprintf( command, "cat %s >> %s", MOVESFILE, TEMPFILE );
		system( command );
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "\n" );
		fclose( tempfile );
	}

	if( phase >= 7 )
	{
		sprintf( command, "cat %s >> %s", CONFLICTSFILE, TEMPFILE );
		system( command );
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "\n" );
		fclose( tempfile );
	}

	if( phase >= 9 )
	{
		sprintf( command, "cat %s >> %s", CITIESFILE, TEMPFILE );
		system( command );
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "\n" );
		fclose( tempfile );
	}

	list_map( NULL, 0 );

	tempfile = fopen( TEMPFILE, "a" );
	fprintf( tempfile, "\n" );
	fprintf( tempfile, "Civilization Cards\n" );
	fprintf( tempfile, "==================\n" );
	fclose( tempfile );

	list_civcards( TEMPFILE, 0 );

	tempfile = fopen( TEMPFILE, "a" );
	fprintf( tempfile, "\n" );
	fprintf( tempfile, "Trade Card Count\n" );
	fprintf( tempfile, "================\n" );
	fclose( tempfile );

	list_cardcount( 0 );

	tempfile = fopen( TEMPFILE, "a" );
	fprintf( tempfile, "\n" );
	fclose( tempfile );

	list_cards( 0 );

	tempfile = fopen( TEMPFILE, "a" );
	fprintf( tempfile, "\n" );
	fclose( tempfile );

	if( phase == 10 )
	{
		write_proposals( 2, logged_in );
	}

	if( phase >= 10 )
	{
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "\n" );
		fclose( tempfile );
		list_trades( 0 );
		tempfile = fopen( TEMPFILE, "a" );
		fprintf( tempfile, "\n" );
		fclose( tempfile );
	}

	list_ast( 0 );

	send_file( TEMPFILE, "game status", sender );
}

print_final_summary()
{
	FILE *output;
	char filename[ 128 ];
	char command[ 128 ];
	int civ;

	sprintf( filename, "%d/%02d05", Turn, Turn );
	output = fopen( filename, "w+" );
	fprintf( output, "Turn %2d - Game Summary\n\n", Turn );

	if( Years[ Turn ] < 0 )
		fprintf( output, "Headlines   %4d B.C.\n", -Years[ Turn ] );
	else
		fprintf( output, "Headlines   %4d A.D.\n", Years[ Turn ] );
	fprintf( output, "=====================\n\n" );
	/*
	fprintf( output,
		"1. ZEUS takes over summary! No more headlines for now!\n" );
	*/

	print_standings( output );

	print_general_info( output, Turn + 1 );
	fprintf( output, "\n" );

	print_card_purchases( output );
	fprintf( output, "\n" );

	fprintf( output, "Civilization Advances\n" );
	fprintf( output, "=====================\n" );
	for( civ = 0; civ < NumCivs; civ++ )
		if( Civ[ civ ].active )
			print_civcards( civ, output );
	fprintf( output, "\n" );

	fprintf( output, "Growth\n" );
	fprintf( output, "======\n\n" );
	fclose( output );
	sprintf( command, "cat %s >> %s", GROWTHFILE, filename );
	system( command );
	output = fopen( filename, "a" );
	fprintf( output, "\n" );

	print_mapboard( output, 6, 0 );
	fclose( output );
}

update_ast_and_techs()
{
	FILE *fileptr;
	int civ, number;

	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			Civ[ civ ].astnew = ast_advance( civ );

			for( number = 0; number < NTCards; number++ )
				if( Card[ number ].cal &&
					( Civ[ civ ].cards[ number ] ) )
						Civ[ civ ].ncards--;
		}
	}

	create_AST_file( ASTNEWFILE, Turn );

	if( !( fileptr = fopen( ASTPOSNEWFILE, "w+" ) ) )
		fopen_error( ASTPOSNEWFILE );
	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			fprintf( fileptr, "%s %d\n", Civ[ civ ].abbr,
				Civ[ civ ].astpos + Civ[ civ ].astnew );
		}
	}
	fclose( fileptr );

	if( !( fileptr = fopen( TECHSNEWFILE, "w+" ) ) )
		fopen_error( TECHSNEWFILE );
	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active )
		{
			fprintf( fileptr, "%s:", Civ[ civ ].abbr );
			for( number = 0; number < NUM_TECHS; number++ )
				if( Civ[ civ ].tech[ number ] )
					fprintf( fileptr, " %s", Tech[ number ].abbr );
			fprintf( fileptr, "\n" );
		}
	}
	fclose( fileptr );

	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active )
		{
			if( !( fileptr = fopen( CARDSNEWFILE( Civ[ civ ].abbr ), "w+" ) ) )
				fopen_error( CARDSNEWFILE( Civ[ civ ].abbr ) );

			fprintf( fileptr, "Trade Cards - %-7s\n", Civ[ civ ].name );
			fprintf( fileptr, "=====================\n\n" );
			fprintf( fileptr, "=========\n" );
			fprintf( fileptr, " Turn %-2d\n", Turn + 1 );
			fprintf( fileptr, "=========\n" );
			fprintf( fileptr, "hnd:" );
			for( number = 0; number < NTCards; number++ )
			{
				if( !Card[ number ].cal && Civ[ civ ].cards[ number ] )
				{
					if( Civ[ civ ].cards[ number ] == 1 )
						fprintf( fileptr, " %s", Card[ number ].abbr );
					else
						fprintf( fileptr, " %d-%s",
							Civ[ civ ].cards[ number ], Card[ number ].abbr );
				}
			}
			fprintf( fileptr, "\n" );
			fprintf( fileptr, "---\n" );

			fclose( fileptr );
		}
	}
}

init_order_file( filename, order )
char *filename;
int order;
{
	FILE *fileptr;
	int civ;

	fileptr = fopen( filename, "w+" );

	if( order == 1 )
	{
		for( civ = 0; civ < NumPlayers; civ++ )
		{
			fprintf( fileptr, "%s\n", Civ[ CensusOrder[ civ ] ].name );
			fprintf( fileptr, "----------\n\n" );
		}
	}
	else if( order == 0 )
	{
		for( civ = 0; civ < NumCivs; civ++ )
		{
			if( Civ[ civ ].active == 1 )
			{
				fprintf( fileptr, "%s\n", Civ[ civ ].name );
				fprintf( fileptr, "----------\n\n" );
			}
		}
	}
	else if( order == 2 )
	{
		for( civ = 0; civ < NumCivs; civ++ )
		{
			if( Civ[ civ ].active == 1 )
			{
				fprintf( fileptr, "%s\n", Civ[ civ ].name );
				fprintf( fileptr, "----------\n" );
				fprintf( fileptr, "r\n" );
				fprintf( fileptr, "e\n\n" );
			}
		}
	}

	fclose( fileptr );
}
