
#include "zeus.h"

int num_tokens( civ )
int civ;
{
	int area;
	int ntokens;
	ntokens = 0;
	for( area = 0; area < NumAreas; area++ )
		ntokens += Map[ area ].tokens[ civ ];
	return( ntokens );
}

int area_has_port( area )
int area;
{
	int index;
	for( index = 0; index < NumAreas; index++ )
	{
		if( ( Map[ index ].geo == PORT ) && 
			( Map[ index ].port == area ) )
			return( 1 );
	}
	return( 0 );
}

int units_in_area( civ, area )
int civ, area;
{
	if( Map[ area ].city == ( civ + 1 ) )
		return( 5 );
	else if( Map[ area ].tokens[ civ ] )
		return( Map[ area ].tokens[ civ ] );
	else
		return( 0 );
}

int area_to_num( name )
char *name;
{
	int area;
	for( area = 0; area < NumAreas; area++ )
		if( !strcasecmp( Map[ area ].abbr, name ) )
			return( area );
	for( area = 0; area < NumAreas; area++ )
		if( !strcasecmp( Map[ area ].name, name ) )
			return( area );
	fprintf( returnfile, ">>> unknown area: \"%s\"\n", name );
	return( -1 );
}

reduce_surplus_population()
{
	FILE *fileptr;
	int area, civ, surplus;

	SurplusPopulation = 0;

	if( !( fileptr = fopen( CITIESFILE, "a" ) ) )
		fopen_error( CITIESFILE );

	fprintf( fileptr, "\nSurplus Population\n==================\n" );

	for( civ = 0; civ < NumCivs; civ++ )
	{
		for ( area = 0; area < NumAreas; area++ )
		{
			if( Map[ area ].tokens[ civ ] >
				( Map[ area ].pop + Civ[ civ ].agr ) )
			{
				surplus = Map[ area ].tokens[ civ ] -
					( Map[ area ].pop + Civ[ civ ].agr );

				Map[ area ].tokens[ civ ] -= surplus;
				/*
				Civ[ civ ].census -= surplus;
				Civ[ civ ].stock  += surplus;
				*/
				fprintf( fileptr, "\n---%s\n",Map[ area ].name );

				if( surplus == 1 )
				{
					fprintf( fileptr, "%s lost %d token\n",
						Civ[ civ ].name, surplus );
					SurplusPopulation = 1;
				}
				else
				{
					fprintf( fileptr, "%s lost %d tokens\n",
						Civ[ civ ].name, surplus );
					SurplusPopulation = 1;
				}
			}
		}
	}
	if( !SurplusPopulation )
		fprintf( fileptr, "None\n" );

	fclose( fileptr );
}

check_for_city_support()
{
	int civ;

	if( mail )
		return;

	printf( "\nchecking for city support...\n\n" );
	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			printf( "%-12s: %d cities : %d tokens - ",
				Civ[ civ ].name, Civ[ civ ].cities, Civ[ civ ].census );
			if( ( Civ[ civ ].census / 2 ) < Civ[ civ ].cities )
			{
				printf( "some cities cannot be supported!\n" );
			}
			else
			{
				printf( "all cities supported\n" );
			}
		}
	}
}

get_map()
{
	FILE *fileptr;
	char line[ 128 ];
	char name[ 32 ], code[ 16 ], abbr[ 8 ];
	int index;
	int nchar;
	int area, civ;

	if( !( fileptr = fopen( MAPFILE, "r" ) ) )
		fopen_error( MAPFILE );

	while( fgetline( fileptr, line, 128 ) != -1 )
	{
		for( index = 0; index < strlen( line ); index++ )
			if( line[ index ] == ',' )
				line[ index ] = ' ';
		sscanf( line, "%s %s %s", name, code, abbr );

		for( area = 0; area < NumAreas; area++ )
		{
			if( !strcmp( name, Map[ area ].name ) )
			{
				printf("error on line %d: \"%s\" previously defined in line %d\n",
					NumAreas+1,name,area+1);
				exit( 1 );
			}
			else if( !strcmp( abbr, Map[ area ].abbr ) )
			{
				printf("error on line %d: \"%s\" previously defined in line %d\n",
					NumAreas+1,abbr,area+1);
				exit( 1 );
			}
		}

		Map[ NumAreas ].name = (char *)malloc( strlen( name ) + 1 );
		strcpy( Map[ NumAreas ].name, name );
		Map[ NumAreas ].abbr = (char *)malloc( strlen( abbr ) + 1 );
		strcpy( Map[ NumAreas ].abbr, abbr );
		/*
		Map[ NumAreas ].active = 1;
		*/
		Map[ NumAreas ].active = 0;

		if( code[ 0 ] == 'S' )
			Map[ NumAreas ].geo = SEA;
		else if( code[ 0 ] == 'C' )
			Map[ NumAreas ].geo = COAST;
		else if( code[ 0 ] == 'I' )
			Map[ NumAreas ].geo = INLAND;
		else if( code[ 0 ] == 'P' )
			Map[ NumAreas ].geo = PORT;
		else
			printf("error in line %d: invalid geo\n",NumAreas + 1);

		if( ( Map[ NumAreas ].geo == COAST ) ||
			( Map[ NumAreas ].geo == INLAND ) )
		{
			Map[ NumAreas ].pop = atoi( &code[ 1 ] );
			for( nchar = 2; nchar < strlen( code ); nchar++ )
				if( code[ nchar ] == 'C' )
					Map[ NumAreas ].site = 1;
				else if( code[ nchar ] == 'c' )
					Map[ NumAreas ].site = 2;
				else if( code[ nchar ] == 'f' )
				{
					nchar++;
					Map[ NumAreas ].flood = atoi( &code[ nchar ] );
				}
				else if( code[ nchar ] == 'v' )
				{
					nchar++;
					Map[ NumAreas ].volcano = atoi( &code[ nchar ] );
				}
				else if( code[ nchar ] == '\\' )
				{
					nchar++;
					for( civ = 0; civ < NumCivs; civ++ )
						if( !strcmp( &code[ nchar ], Civ[ civ ].abbr ) )
						{
							Map[ NumAreas ].start = civ + 1;
							break;
						}
				}
		}
		else if( ( Map[ NumAreas ].geo == PORT ) )
		{
			for( nchar = 0; nchar < strlen( code ); nchar++ )
				if( code[ nchar ] == '\\' )
				{
					nchar++;
					for( area = 0; area < NumAreas; area++ )
						if( !strcmp( &code[ nchar ], Map[ area ].abbr ) )
						{
							Map[ NumAreas ].port = area;
							break;
						}
				}
		}
		NumAreas++;
	}

	fclose( fileptr );
}

get_paths()
{
	FILE *fileptr;
	char line[ 128 ];
	char *word[ 65 ];
	int index, nwrd;
	int area, path;
	int nline;
	int narea;

	if( !( fileptr = fopen( PATHSFILE, "r" ) ) )
		fopen_error( PATHSFILE );

	nline = 0;
	while( fgetline( fileptr, line, 128 ) != -1 )
	{
		nline++;

		index = 0;
		word[ index++ ] = strtok( line, " -:," );
		while( ( word[ index ] = strtok( NULL, " -:," ) ) ) index++;

		area = area_to_num( word[0] );
		if( area < 0 )
		{
			printf("bad area abbrev: line %d\n",nline);
			printf("abbrev is %s\n",word[0]);
			exit( 1 );
		}

		if( !Map[ area ].active )
			continue;

		/* sea path */
		if( !strcmp( word[1], "se" ) )
		{
			Map[ area ].nsea = index - 2;
			Map[ area ].sea = (short *)calloc( Map[ area ].nsea,
				sizeof(short) );
			narea = 0;
			for( nwrd = 2; nwrd < index; nwrd++ )
			{
				path = area_to_num( word[ nwrd ] );
				if( path < 0 )
				{
					printf("bad area abbrev: line %d\n",nline);
					printf("abbrev is %s\n",word[ nwrd ]);
					exit( 1 );
				}
				if( Map[ path ].active )
				{
					*(Map[ area ].sea + narea) = path;
					narea++;
				}
				else
					Map[ area ].nsea--;
			}
		}
		/* land path */
		else if( !strcmp( word[1], "la" ) )
		{
			Map[ area ].nland = index - 2;
			Map[ area ].land = (short *)calloc( Map[ area ].nland,
				sizeof(short) );
			narea = 0;
			for( nwrd = 2; nwrd < index; nwrd++ )
			{
				path = area_to_num( word[ nwrd ] );
				if( path < 0 )
				{
					printf("bad area abbrev: line %d\n",nline);
					printf("abbrev is %s\n",word[ nwrd ]);
					exit( 1 );
				}
				if( Map[ path ].active )
				{
					*(Map[ area ].land + narea) = path;
					narea++;
				}
				else
					Map[ area ].nland--;
			}
		}
	}

	fclose( fileptr );
}

verify_paths()
{
	int area, path, index;
	int npath;
	int found;

	for( area = 0; area < NumAreas; area++ )
	{
		for( npath = 0; npath < Map[ area ].nsea; npath++ )
		{
			path = *(Map[ area ].sea + npath);
			found = 0;
			for( index = 0; index < Map[ path ].nsea; index++ )
			{
				if( *(Map[ path ].sea + index) == area )
				{
					found = 1;
					break;
				}
			}
			if( !found )
			{
				printf("path inconsistency found!\n%s-se %s\n",
					Map[area].abbr,Map[path].abbr);
				exit( 1 );
			}
		}
	}

	for( area = 0; area < NumAreas; area++ )
	{
		for( npath = 0; npath < Map[ area ].nland; npath++ )
		{
			path = *(Map[ area ].land + npath);
			found = 0;
			for( index = 0; index < Map[ path ].nland; index++ )
			{
				if( *(Map[ path ].land + index) == area )
				{
					found = 1;
					break;
				}
			}
			if( !found )
			{
				printf("path inconsistency found!\n%s-la %s\n",
					Map[area].abbr,Map[path].abbr);
				exit( 1 );
			}
		}

	}
}

get_latest_mapboard()
{
	int phase;

	get_mapboard();

	phase = 2 * Phase + PhaseComplete;
	if( phase >= 15 )
		handle_growth();
	else if( phase >= 13 )
		handle_calamities();
	else if( phase >= 11 )
		handle_deal();
	else if( phase >= 9 )
		handle_cities();
	else if( phase >= 7 )
		handle_conflict();
	else if( phase >= 4 )
		handle_move();
	else if( phase >= 3 )
		handle_ships();
}

get_mapboard()
{
	FILE *fileptr;
	char line[ 128 ];
	int index;
	int curciv;
	char *word[ 65 ];
	int area;
	int civ;
	int nwrd;
	int nline = 0;

	if( !( fileptr = fopen( MAPBOARDFILE, "r" ) ) )
		fopen_error( MAPBOARDFILE );

	while( fgetline( fileptr, line, 128 ) != -1 )
	{
		nline++;
		index = 0;
		word[ index++ ] = strtok( line, " -:," );
		while( ( word[ index ] = strtok( NULL, " -:," ) ) ) index++;

		if( !word[ 0 ] )
			continue;

		for( civ = 0; civ < NumCivs; civ++ )
		{
			if( !strcmp( word[ 0 ], Civ[ civ ].name ) )
			{
				curciv = civ;
				Civ[ curciv ].cities = 0;
				Civ[ curciv ].ships = 0;
				for( area = 0; area < NumAreas; area++ )
				{
					Map[ area ].tokens[ curciv ] = 0;
					Map[ area ].ships[ curciv ] = 0;
					Map[ area ].oships[ curciv ] = 0;
					if( ( Map[ area ].city == ( curciv + 1 ) ) )
						Map[ area ].city = 0;
				}
				break;
			}
		}

		/* cities */
		if( !strcmp( word[ 0 ], "C" ) )
		{
			for( nwrd = 1; nwrd < index; nwrd++ )
			{
				area = area_to_num( word[ nwrd ] );
				if( area == -1 )
				{
					fprintf( returnfile,
						">>> error in mapboard file: line %d\n", nline );
					zeus_exit( "mapboard error" );
				}
				Map[ area ].city = curciv + 1;
				Civ[ curciv ].cities++;
			}
		}
		/* ships */
		else if( !strcmp( word[ 0 ], "S" ) )
		{
			for( nwrd = 1; nwrd < index; nwrd++ ) {
				area = area_to_num( word[ nwrd ] );
				if( area == -1 )
				{
					fprintf( returnfile,
						">>> error in mapboard file: line %d\n", nline );
					zeus_exit( "mapboard error" );
				}
				Map[ area ].oships[ curciv ]++;
			}
		}
		/* tokens */
		else if( ( strcmp( word[ 0 ], "0" ) > 0 ) &&
			( strcmp( word[ 0 ], ":" ) < 0 ) )
		{
			if( index != 2 )
			{
				fprintf( returnfile,
					">>> error in mapboard file: line %d\n", nline );
				zeus_exit( "mapboard error" );
			}
			area = area_to_num( word[ 1 ] );
			if( area == -1 )
			{
				fprintf( returnfile,
					">>> error in mapboard file: line %d\n", nline );
				zeus_exit( "mapboard error" );
			}
			Map[ area ].tokens[ curciv ] = atoi( word[0] );
		}
		/* other info */
		else if( !strcmp( word[ 0 ], "*" ) )
		{
			Civ[ curciv ].census   = atoi( word[ 3 ] );
			Civ[ curciv ].treasury = atoi( word[ 5 ] );
			Civ[ curciv ].stock    = atoi( word[ 7 ] );
		}
	}

	fclose( fileptr );

	get_census_order();
}

do_ship_movement( nline, word, index, curciv )
int nline;
char *word[ 65 ];
int index;
int curciv;
{
	int area[ 6 ];
	int ntokens;
	int num;
	int legal_path;
	int nwrd;
	int lastarea;
	char  action, pass;
	int passengers;
	int maxmove;
	int dock;

	if( Civ[ curciv ].clo )
		maxmove = SHIP_MOVE + 1;
	else
		maxmove = SHIP_MOVE;

	passengers = 0;
	for( nwrd = 1; nwrd < index; nwrd++ )
	{
		if( nwrd == 1 )
		{
			area[ 0 ] = area_to_num( word[ 1 ] );
			if( area[ 0 ] == -1 )
			{
				fprintf( returnfile,
					">>> illegal move from \"%s\": line %d\n",
					word[ 1 ], nline );
				zeus_exit( "movement error" );
			}

			if( !Map[ area[ 0 ] ].ships[ curciv ] )
			{
				fprintf( returnfile,
					">>> no ship in %s: line %d\n",
					Map[ area[ 0 ] ].name, nline );
				zeus_exit( "movement error" );
			}
			Map[ area[ 0 ] ].ships[ curciv ]--;
			lastarea = 0;
		}
		else if( !strncmp( word[ nwrd ], "(", 1 ) ||
			!strncmp( word[ nwrd ], "*", 1 ) )
		{
			if( !strncmp( word[ nwrd ], "(", 1 ) )
			{
				sscanf( word[ nwrd ], "(%c%c", &action, &pass );
			}
			else if( !strncmp( word[ nwrd ], "*", 1 ) )
			{
				sscanf( word[ nwrd ], "*%c%c", &action, &pass );
			}

			ntokens = pass - '0';
			if( Map[ area[ lastarea ] ].geo == PORT )
				dock = Map[ area[ lastarea ] ].port;
			else
				dock = area[ lastarea ];
			if( ( action == 'E' ) || ( action == 'e' ) )
			{
				if( ( ntokens + passengers ) > SHIP_CAPACITY )
				{
					fprintf( returnfile,
						">>> ship capacity exceeded: line %d\n",
						nline );
					fprintf( returnfile,
						">>> area is %s\n",
						Map[ area[ lastarea ] ].name );
					zeus_exit( "movement error" );
				}
				else if( ntokens > Map[ dock ].tokens[ curciv ] )
				{
					fprintf( returnfile,
						">>> not enough tokens to embark: line %d\n",
						nline );
					fprintf( returnfile,
						">>> area is %s\n",
						Map[ area[ lastarea ] ].name );
					zeus_exit( "movement error" );
				}
				Map[ dock ].tokens[ curciv ] -= ntokens;
				passengers += ntokens;
			}
			else if( ( action == 'D' ) || ( action == 'd' ) )
			{
				if( ntokens > passengers )
				{
					fprintf( returnfile,
						">>> not enough tokens to disembark: line %d\n",
						nline );
					fprintf( returnfile,
						">>> area is %s\n",
						Map[ area[ lastarea ] ].name );
					zeus_exit( "movement error" );
				}
				Map[ dock ].dtok += ntokens;
				passengers -= ntokens;
			}
			else
			{
				fprintf( returnfile,
					">>> syntax error: line %d\n", nline );
				zeus_exit( "movement error" );
			}
		}
		else
		{
			area[ lastarea + 1 ] = area_to_num( word[ nwrd ] );
			if( area[ lastarea + 1 ] == -1 )
			{
				fprintf( returnfile,
					">>> illegal move to \"%s\": line %d\n",
					word[ nwrd ], nline );
				zeus_exit( "movement error" );
			}
			if( ( lastarea + 1 ) > maxmove )
			{
				fprintf( returnfile,
					">>> ship exceeded movement allowance: line %d\n",
					nline );
				zeus_exit( "movement error" );
			}
			legal_path = 0;
			for( num = 0; num < Map[ area[ lastarea ] ].nsea; num++ )
			{
				if( *(Map[ area[ lastarea ] ].sea + num) ==
					area[ lastarea + 1 ] )
				{
					if( ( !Civ[ curciv ].ast &&
						Map[ area[ lastarea + 1 ] ].geo != SEA ) ||
						Civ[ curciv ].ast )
					{
						legal_path = 1;
						lastarea++;
						break;
					}
				}
			}
			if( !legal_path )
			{
				fprintf( returnfile,
					">>> no path to \"%s\": line %d\n",
					Map[ area[ lastarea + 1 ] ].name, nline );
				zeus_exit( "movement error" );
			}
		}
	}
	if( Map[ area[ lastarea ] ].geo == SEA )
	{
		fprintf( returnfile,
			">>> ship cannot end move at sea: line %d\n",
			nline );
		fprintf( returnfile,
			">>> area is %s\n",
			Map[ area[ lastarea ] ].name );
		zeus_exit( "movement error" );
	}
	if( passengers > 0 )
		Map[ area[ lastarea ] ].dtok += passengers;
	Map[ area[ lastarea ] ].dshp++;
}

int do_road_movement( curciv, area_num, ntokens, nline )
int curciv;
int area_num[ 6 ];
int ntokens;
int nline;
{
	int area, civ;

	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( ( civ != curciv ) && units_in_area( civ, area_num[ 1 ] ) )
		{
			fprintf( returnfile,
				">>> opponent's units block road: line %d\n",
				nline );
			fprintf( returnfile,
				">>> area is %s\n",
				Map[ area_num[ 1 ] ].name );
			zeus_exit( "movement error" );
		}
	}

	for( area = 0; area < Map[ area_num[ 1 ] ].nland; area++ )
	{
		if( *(Map[ area_num[ 1 ] ].land + area) == area_num[ 2 ] )
		{
			Map[ area_num[ 0 ] ].tokens[ curciv ] -= ntokens;
			Map[ area_num[ 2 ] ].dtok += ntokens;
			return( 1 );
		}
	}
	return( 0 );
}

get_moves( input, output )
FILE *input;
FILE *output;
{
	FILE *movefile;
	char line[ 128 ];
	int index;
	int curciv;
	char *word[ 65 ];
	int area_num[ 6 ];
	int ntokens;
	int nline;
	int civ, area;
	int legal_path;

	if( input == NULL )
	{
		if( !( movefile = fopen( MOVESFILE, "r" ) ) )
			fopen_error( MOVESFILE );
	}
	else
		movefile = input;

	curciv = -1;
	nline = 0;
	while( fgetline( movefile, line, 128 ) != -1 )
	{
		if( output != NULL )
			fprintf( output, "%s\n", line );

		if( mail && ( input == stdin ) )
		{
			fprintf( maillog, "%s\n", line );
			fprintf( returnfile, "%s\n", line );
			curciv = logged_in;
		}

		nline++;
		index = 0;
		word[ index++ ] = strtok( line, " :," );
		while( ( word[ index ] = strtok( NULL, " :," ) ) ) index++;

		if( !word[ 0 ] )
			continue;

		for( civ = 0; civ < NumCivs; civ++ )
		{
			if( !strcasecmp( word[ 0 ], Civ[ civ ].name ) )
			{
				if( curciv >= 0 )
				{
					for( area = 0; area < NumAreas; area++ )
					{
						Map[ area ].tokens[ curciv ] += Map[ area ].dtok;
						Map[ area ].ships[ curciv ] += Map[ area ].dshp;
					}
				}
				if( !mail )
				{
					if( curciv >= 0 )
					{
						if( Civ[ curciv ].moved )
							printf( "done\n" );
						else
							printf( "\n" );
					}
					printf( "%s...", Civ[ civ ].name );
				}
				curciv = civ;
				for( area = 0; area < NumAreas; area++ )
				{
					Map[ area ].dtok = 0;
					Map[ area ].dshp = 0;
				}
				break;
			}
		}

		if( ( strcmp( word[ 0 ], "0" ) > 0 ) &&
			( strcmp( word[ 0 ], ":" ) < 0 ) )
		{
			if( !Civ[ curciv ].roa && ( index != 3 ) )
			{
				fprintf( returnfile,
					">>> cannot move tokens two areas: line %d\n",
					nline );
				fprintf( returnfile,
					">>> civilization does not hold roadbuilding\n" );
				zeus_exit( "movement error" );
			}

			ntokens = atoi( word[ 0 ] );

			area_num[ 0 ] = area_to_num( word[ 1 ] );
			if( area_num[ 0 ] == -1 )
			{
				fprintf( returnfile,
					">>> illegal move from \"%s\": line %d\n",
					word[ 1 ], nline );
				zeus_exit( "movement error" );
			}

			area_num[ 1 ] = area_to_num( word[ 2 ] );
			if( area_num[ 1 ] == -1 )
			{
				fprintf( returnfile,
					">>> illegal move to \"%s\": line %d\n",
					word[ 2 ], nline );
				zeus_exit( "movement error" );
			}

			/* third area for road movement */
			if( Civ[ curciv ].roa && ( index == 4 ) )
			{
				area_num[ 2 ] = area_to_num( word[ 3 ] );
				if( area_num[ 2 ] == -1 )
				{
					fprintf( returnfile,
						">>> illegal move to \"%s\": line %d\n",
						word[ 3 ], nline );
					zeus_exit( "movement error" );
				}
			}

			legal_path = 0;
			if( ntokens <= Map[ area_num[ 0 ] ].tokens[ curciv ] )
			{
				if( index == 3 )
				{
					for( area = 0; area < Map[ area_num[ 0 ] ].nland; area++ )
					{
						if( *(Map[ area_num[ 0 ] ].land + area) ==
							area_num[ 1 ] )
						{
							Map[ area_num[ 0 ] ].tokens[ curciv ] -= ntokens;
							Map[ area_num[ 1 ] ].dtok += ntokens;
							legal_path = 1;
							break;
						}
					}
					if( !legal_path )
					{
						fprintf( returnfile,
							">>> no path to \"%s\": line %d\n",
							Map[ area_num[ 1 ] ].name, nline );
						zeus_exit( "movement error" );
					}
				}
				/* do road movement */
				else if( index == 4 )
				{
					legal_path =
						do_road_movement( curciv, area_num, ntokens, nline );
					if( !legal_path )
					{
						fprintf( returnfile,
							">>> no path to \"%s\": line %d\n",
							Map[ area_num[ 2 ] ].name, nline );
						zeus_exit( "movement error" );
					}
				}
			}
			else
			{
				fprintf( returnfile,
					">>> not enough tokens to move: line %d\n",
					nline );
				fprintf( returnfile,
					">>> area is %s\n",
					Map[ area_num[ 0 ] ].name );
				zeus_exit( "movement error" );
			}
			Civ[ curciv ].moved = 1;
		}
		else if( !strcmp( word[ 0 ], "S" ) )
		{
			if( index < 3 )
			{
				fprintf( returnfile,
					">>> ship move order error: line %d\n", nline );
				zeus_exit( "movement error" );
			}
			do_ship_movement( nline, word, index, curciv );
			Civ[ curciv ].moved = 1;
		}
		else if( !strcmp( word[ 0 ], "none" ) )
		{
			Civ[ curciv ].moved = 1;
		}
		else if( !strcmp( word[ 0 ], "pass" ) )
		{
			Civ[ curciv ].moved = 1;
		}
		else if( !strcmp( word[ 0 ], "end" ) )
		{
			break;
		}
		else if( !strcmp( word[ 0 ], "exit" ) )
		{
			break;
		}

	}
	if( curciv >= 0 )
	{
		for( area = 0; area < NumAreas; area++ )
		{
			Map[ area ].tokens[ curciv ] += Map[ area ].dtok;
			Map[ area ].ships[ curciv ] += Map[ area ].dshp;
		}
		if( !mail )
		{
			if( Civ[ curciv ].moved )
				printf( "done\n" );
			else
				printf( "\n" );
		}
	}
	if( movefile != stdin )
		fclose( movefile );
}

handle_loot( area, civ, city_civ, output )
int area, civ, city_civ;
FILE *output;
{
	FILE *fileptr;
	int random[ HAND_SIZE ];
	int card, index;

	if( !( fileptr = fopen( LOOTFILE, "a" ) ) )
		fopen_error( LOOTFILE );

	if( Civ[ city_civ ].ncards > 0 )
	{
		fprintf( output, "%s loots trade card from %s\n",
			Civ[ civ ].name, Civ[ city_civ ].name );

		/* determine which card is looted */
		index = 0;
		get_numbers( Civ[ city_civ ].ncards, random );
		for( card = 0; card < NTCards; card++ )
		{
			index += Civ[ city_civ ].cards[ card ];
			if( random[ 0 ] < index )
			{
				fprintf( fileptr, "%s:%s loots %s from %s\n",
					Map[ area ].name, Civ[ civ ].name,
					Card[ card ].abbr, Civ[ city_civ ].name );
				fprintf( fileptr, "loot-%s: %s\n",
					Civ[ city_civ ].abbr, Card[ card ].abbr );
				fprintf( fileptr, "lost-%s: %s\n",
					Civ[ civ ].abbr, Card[ card ].abbr );
				fprintf( fileptr, "---\n" );
				Civ[ city_civ ].cards[ card ]--;
				Civ[ city_civ ].ncards--;
				Civ[ civ ].newcards[ Civ[ civ ].ndraws ] = card;
				Civ[ civ ].ndraws++;
				break;
			}
		}
	}

	fclose( fileptr );
}

int siege( area, output )
int area;
FILE *output;
{
	int civ;
	int do_siege;
	int city_civ;
	int city_siz;
	int ntokens;

	do_siege = 0;
	if( Map[ area ].city )
	{
		city_civ = Map[ area ].city - 1;
		for( civ = 0; civ < NumCivs; civ++ )
		{
			if( ( Map[ area ].tokens[ civ ] ) &&
				( civ != city_civ ) )
			{
				city_siz = 6 - Civ[ civ ].eng + Civ[ city_civ ].eng;
				if( Map[ area ].tokens[ civ ] > city_siz )
				{
					if( Civ[ city_civ ].stock < city_siz )
					{
						ntokens = Civ[ city_civ ].stock;
							Civ[ city_civ ].stock;
					}
					else
					{
						ntokens = city_siz;
					}
					Civ[ city_civ ].cities--;
					Civ[ city_civ ].stock -= ntokens;
					Map[ area ].tokens[ city_civ ] = ntokens;
					Map[ area ].city = 0;
					fprintf( output,
						"\n---%s Successful Seige\n",Map[ area ].name );
					fprintf( output,
						"%s lost city --> %d tokens\n", 
							Civ[ city_civ ].name, ntokens );

					handle_loot( area, civ, city_civ, output );
					do_siege = 1;
				}
				else
				{
					fprintf( output,
						"\n---%s Unsuccessful Seige\n",Map[ area ].name );
					fprintf( output,
						"\n---%s\n",Map[ area ].name );
					fprintf( output,
						"%s lost %d tokens\n",
						Civ[civ].name, Map[ area ].tokens[ civ ] );
					Map[ area ].tokens[ civ ] = 0;
				}
				break;
			}
		}
	}

	return( do_siege );
}

do_token_conflict( area, output )
int area;
FILE *output;
{
	int civ;
	int total_pop;
	int total_civ;
	int num_civ;
	int res_civ[ MAX_CIVS ], ord_civ[ MAX_CIVS ];
	int ntokens;
	int tok_lost[ MAX_CIVS ];

	total_civ = 0;
	total_pop = 0;
	for( civ = 0; civ < NumCivs; civ++ )
		tok_lost[ civ ] = 0;
	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Map[ area ].tokens[ civ ] )
		{
			res_civ[ total_civ ] = civ;
			ord_civ[ total_civ ] = Map[ area ].tokens[ civ ] +
				( Civ[ res_civ[ total_civ ] ].met * MaxTokens );
				
			total_pop += Map[ area ].tokens[ civ ];
			total_civ++;
		}
	}
	num_civ = total_civ;
	ntokens = 0;
	while( ( num_civ > 1 ) &&
		( ( total_pop > Map[ area ].pop ) || Map[ area ].city ) )
	{
		if( ntokens == 0 )
			fprintf( output, "\n---%s\n",Map[ area ].name );
		for( civ = 0; civ < total_civ; civ++ )
		{
			if( ord_civ[ civ ] == ntokens )
			{
				if( Map[ area ].tokens[ res_civ[ civ ] ] )
				{
					Map[ area ].tokens[ res_civ[ civ ] ]--;
					tok_lost[ res_civ[ civ ] ]++;
					total_pop--;
					if( Map[ area ].tokens[ res_civ[ civ ] ] == 0 )
						num_civ--;
					Civ[ res_civ[ civ ] ].census--;
					Civ[ res_civ[ civ ] ].stock++;
				}
			}
		}
		if( ntokens == ( 2 * MaxTokens ) )
			ntokens = 0;
		ntokens++;
	}
	for( civ = 0; civ < total_civ; civ++ )
		if( tok_lost[ res_civ[ civ ] ] == 1 )
		{
			fprintf( output, "%s lost %d token\n",
				Civ[ res_civ[ civ ] ].name, tok_lost[ res_civ[ civ ] ]);
			ConflictOccured = 1;
		}
		else if( tok_lost[ res_civ[ civ ] ] > 1 )
		{
			fprintf( output, "%s lost %d tokens\n",
				Civ[ res_civ[ civ ] ].name, tok_lost[ res_civ[ civ ] ]);
			ConflictOccured = 1;
		}
	for( civ = 0; civ < total_civ; civ++ )
		if( !strcmp( Civ[ civ ].abbr, "pir" ) )
			if( Map[ area ].tokens[ civ ] > 0 )
			{
				fprintf( output, "%d Pirate tokens removed\n",
					Map[ area ].tokens[ civ ] ); 
				Civ[ civ ].census -= Map[ area ].tokens[ civ ];
				Civ[ civ ].stock += Map[ area ].tokens[ civ ];
				Map[ area ].tokens[ civ ] = 0;
			}
}

do_city_conflicts( fileptr )
FILE *fileptr;
{
	int random[ MAX_AREAS ];
	int area, index, pop;

	/* city conflicts - lowest pop first then random */
	get_numbers( NumAreas, random );
	for( pop = 1; pop <= 5; pop++ )
	{
		for( index = 0; index < NumAreas; index++ )
		{
			area = random[ index ];
			if( Map[ area ].pop == pop )
			{
				if( siege( area, fileptr ) )
					do_token_conflict( area, fileptr );
			}
		}
	}
	update_looting();
}

update_looting()
{
	FILE *fileptr;
	int civ, card;

	if( !( fileptr = fopen( LOOTFILE, "a" ) ) )
		fopen_error( LOOTFILE );

	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( Civ[ civ ].active == 1 )
		{
			for( card = 0; card < Civ[ civ ].ndraws; card++ )
			{
				Civ[ civ ].cards[ Civ[ civ ].newcards[ card ] ]++;
				Civ[ civ ].ncards++;
			}
			fprintf( fileptr, "\n***%-15s", Civ[ civ ].name );
			fprintf( fileptr, "\nhnd: " );
			for( card = 0; card < NTCards; card++ )
			{
				if( Civ[ civ ].cards[ card ] == 1 )
					fprintf( fileptr, "%3s ", Card[ card ].abbr );
				else if( Civ[ civ ].cards[ card ] > 1 )
					fprintf( fileptr, "%1d-%3s ",
						Civ[ civ ].cards[ card ], Card[ card ].abbr );
			}
			fprintf( fileptr, "\n" );
		}
	}

	fclose( fileptr );
}

do_conflicts()
{
	FILE *fileptr;
	int area;

	ConflictOccured = 0;

	/* create loot summary file */
	if( !( fileptr = fopen( LOOTFILE, "w+" ) ) )
		fopen_error( LOOTFILE );
	fclose( fileptr );

	if( !( fileptr = fopen( CONFLICTSFILE, "w+" ) ) )
		fopen_error( CONFLICTSFILE );

	fprintf( fileptr, "Conflicts\n=========\n" );

	/* normal non-city conflicts */
	for( area = 0; area < NumAreas; area++ )
		do_token_conflict( area, fileptr );

	do_city_conflicts( fileptr );
	/*
	for( area = 0; area < NumAreas; area++ )
		if( siege( area, fileptr ) )
			do_token_conflict( area, fileptr );
	*/

	handle_pillage( fileptr );

	if( !ConflictOccured )
		fprintf( fileptr, "None\n" );

	fclose( fileptr );
}

print_empty_areas( fileptr )
FILE *fileptr;
{
	int area, civ, units;
	fprintf( fileptr, "\n\nUnpopulated Areas\n" );
	fprintf( fileptr, "-----------------\n" );
	for ( area = 0; area < NumAreas; area++ )
	{
		units = 0;
		for( civ = 0; civ < NumCivs; civ++ )
		{
			units += units_in_area( civ, area );
		}
		if( !units )
		{
			fprintf( fileptr, "%s\n", Map[ area ].name );
		}
	}
}

print_units( fileptr, area, civ )
FILE *fileptr;
int area, civ;
{
	int civ2;

	fprintf( fileptr, " " );

	if( ( Map[ area ].city ) && ( ( Map[ area ].city - 1 ) != civ ) )
		fprintf( fileptr, "C-%s ", Civ[ Map[ area ].city - 1 ].abbr );

	for( civ2 = 0; civ2 < NumCivs; civ2++ )
	{
		if( civ2 == civ )
			continue;
		if( Map[ area ].tokens[ civ2 ] )
			fprintf( fileptr, "%d-%s ",
				Map[ area ].tokens[ civ2 ], Civ[ civ2 ].abbr );
		if( Map[ area ].ships[ civ2 ] == 1 )
			fprintf( fileptr, "S-%s ",
				Civ[ civ2 ].abbr );
		else if( Map[ area ].ships[ civ2 ] > 1 )
			fprintf( fileptr, "%dS-%s ",
				Map[ area ].ships[ civ2 ], Civ[ civ2 ].abbr );
	}
}

print_mapboard( boardfile, version, info )
FILE *boardfile;
int version;
int info;
{
	int area;
	int civ;
	int cities, ships, nships;
	char temp[ 128 ];
	int area2;

	if( info )
	{
		fprintf( boardfile,
			"*** Mapboard Information - Turn:%d - Phase:%s ***\n\n",
			Turn, PhaseName[ version + 1 ] );
		fprintf( boardfile,
			"Key: <pop> <site> <coast> <volcano> <flood> <start> [units]\n\n" );
		fprintf( boardfile, "\"*\" = Normal City Site\n" );
		fprintf( boardfile, "\"+\" = Floodable City Site\n" );
		fprintf( boardfile, "\"c\" = Coastal Area\n" );
		fprintf( boardfile, "\">\" = Overpopulated Area\n" );
	}
	else
	{
		fprintf( boardfile, "*** Mapboard - Turn:%d - Phase:%s ***\n",
			Turn, PhaseName[ version + 1 ] );
	}

	for( civ = 0; civ < NumCivs; civ++ )
	{
		Civ[ civ ].census = num_tokens( civ );
		Civ[ civ ].stock = MaxTokens - Civ[ civ ].census - Civ[ civ ].treasury;

		if( ( Civ[ civ ].census == 0 ) && ( Civ[ civ ].cities == 0 ) )
			continue;

		fprintf( boardfile, "\n%s\n", Civ[ civ ].name );
		fprintf( boardfile, "-----------\n" );

		/*
		if( info )
		{
			int tech;
			fprintf( boardfile, "advances:" );
			for( tech = 0; tech < NUM_TECHS; tech++ )
				if( Civ[ civ ].tech[ tech ] )
					fprintf( boardfile, " %s", Tech[ tech ].abbr );
			fprintf( boardfile, "\n" );
			fprintf( boardfile,
				"score:    %d total = %d cards + %d ast + %d cities\n",
				total_civcard_points( civ ) +
				( Civ[ civ ].cities * 50 ) +
				( Civ[ civ ].astpos * 100 ),
				total_civcard_points( civ ),
				Civ[ civ ].astpos * 100,
				Civ[ civ ].cities * 50 );
			fprintf( boardfile, "---\n" );
		}
		*/
		/*
		if( info )
		{
			fprintf( boardfile, "Agriculture: " );
			fprintf( boardfile, "%s; ", YesNo[ Civ[ civ ].agr ] );
			fprintf( boardfile, "Metalworking: " );
			fprintf( boardfile, "%s; ", YesNo[ Civ[ civ ].met ] );
			fprintf( boardfile, "Engineering: " );
			fprintf( boardfile, "%s; ", YesNo[ Civ[ civ ].eng ] );
			fprintf( boardfile, "\n" );
		}
		*/

		fprintf( boardfile, "* %d cities : ",Civ[ civ ].cities );
		fprintf( boardfile, "%d census : ",Civ[ civ ].census );
		fprintf( boardfile, "%d treasury : ",Civ[ civ ].treasury );
		fprintf( boardfile, "%d stock\n",Civ[ civ ].stock );

		if( info )
			fprintf( boardfile, "---\n" );

		/* print out ships */
		ships = 0;
		for ( area = 0; area < NumAreas; area++ )
		{
			if( Map[ area ].ships[ civ ] )
			{
				if( !ships )
					fprintf( boardfile, "S  " );
				for( nships = 0; nships < Map[ area ].ships[ civ ]; nships++ )
				{
					fprintf( boardfile, "%s ",Map[ area ].name );
				}
				ships = 1;
			}
		}
		if( ships )
			fprintf( boardfile, "\n" );
		if( info )
		{
			if( !ships )
				fprintf( boardfile, "S  none\n" );
			fprintf( boardfile, "---\n" );
		}

		/* print out cities */
		cities = 0;
		for ( area = 0; area < NumAreas; area++ )
		{
			if( Map[ area ].city == ( civ + 1 ) )
			{
				if( info )
				{
					fprintf( boardfile, "C  %-16s", Map[ area ].name );
					fprintf( boardfile, "  " );
					fprintf( boardfile, " %d", Map[ area ].pop );

					if( Map[ area ].site == 1 )
						fprintf( boardfile, " *" );
					else if( Map[ area ].site == 2 )
						fprintf( boardfile, " +" );
					else
						fprintf( boardfile, " ." );

					if( Map[ area ].geo == 1 )
						fprintf( boardfile, " c" );
					else if( Map[ area ].geo == 2 )
						fprintf( boardfile, " ." );

					if( Map[ area ].volcano )
						fprintf( boardfile, " v%d", Map[ area ].volcano );
					else
						fprintf( boardfile, " .." );

					if( Map[ area ].flood )
						fprintf( boardfile, " f%d", Map[ area ].flood );
					else
						fprintf( boardfile, " .." );

					if( Map[ area ].start )
						fprintf( boardfile, " %3s",
							Civ[ Map[ area ].start - 1 ].abbr );
					else
						fprintf( boardfile, " ..." );

					print_units( boardfile, area, civ );
					fprintf( boardfile, "\n" );
				}
				else
				{
					if( !cities )
						fprintf( boardfile, "C  " );
					fprintf( boardfile, "%s ", Map[ area ].name );
				}
				cities = 1;
			}
		}
		if( !info && cities )
			fprintf( boardfile, "\n" );
		if( info )
			fprintf( boardfile, "---\n" );

		/* print out tokens */
		for ( area = 0; area < NumAreas; area++ )
			if( Map[ area ].tokens[ civ ] )
			{
				if( info )
				{
					fprintf( boardfile, "%-2d %-16s",
						Map[ area ].tokens[ civ ], Map[ area ].name );

					if( Map[ area ].tokens[ civ ] >
						( Map[ area ].pop + Civ[ civ ].agr ) )
						fprintf( boardfile, " >" );
					else
						fprintf( boardfile, "  " );

					fprintf( boardfile, " %d", Map[ area ].pop );

					if( Map[ area ].site == 1 )
						fprintf( boardfile, " *" );
					else if( Map[ area ].site == 2 )
						fprintf( boardfile, " +" );
					else
						fprintf( boardfile, " ." );

					if( Map[ area ].geo == 1 )
						fprintf( boardfile, " c" );
					else if( Map[ area ].geo == 2 )
						fprintf( boardfile, " ." );

					if( Map[ area ].volcano )
						fprintf( boardfile, " v%d", Map[ area ].volcano );
					else
						fprintf( boardfile, " .." );

					if( Map[ area ].flood )
						fprintf( boardfile, " f%d", Map[ area ].flood );
					else
						fprintf( boardfile, " .." );

					if( Map[ area ].start )
						fprintf( boardfile, " %3s",
							Civ[ Map[ area ].start - 1 ].abbr );
					else
						fprintf( boardfile, " ..." );
					print_units( boardfile, area, civ );
				}
				else
					fprintf( boardfile, "%-2d %s",
						Map[ area ].tokens[ civ ], Map[ area ].name );
				fprintf( boardfile, "\n" );
			}
	}
}

do_eliminatecity( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't eliminate city\n" );
		zeus_exit( "command error" );
	}
	Map[ area ].city = 0;
	Civ[ curciv ].cities--;
	fprintf( output, "\tC  %s\n", Map[ area ].name );
}

do_addcity( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't add city\n" );
		zeus_exit( "command error" );
	}
	Map[ area ].city = curciv + 1;
	Civ[ curciv ].cities++;
	fprintf( output, "\tC  %s\n", Map[ area ].name );
}

do_addtoken( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int ntokens;
	int area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't add token\n" );
		zeus_exit( "command error" );
	}
	if( index == 3 )
		ntokens = atoi( word[ 2 ] );
	else
		ntokens = 1;
	Map[ area ].tokens[ curciv ] += ntokens;
	fprintf( output, "\t%-2d %s\n", ntokens, Map[ area ].name );
}

do_eliminatetoken( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int ntokens;
	int area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't eliminate token" );
		zeus_exit( "command error" );
	}
	if( index == 3 )
		ntokens = atoi( word[ 2 ] );
	else
		ntokens = 1;
	Map[ area ].tokens[ curciv ] -= ntokens;
	fprintf( output, "\t%-2d %s\n", ntokens, Map[ area ].name );
}

resolve_calamities( input, output )
FILE *input;
FILE *output;
{
	char line[ 128 ];
	char *word[ 65 ];
	int index, civ, area, curciv, card;
	int ntokens;
	int ncal = 0;
	int count;

	curciv = -1;
	while( fgetline( input, line, 128 ) != -1 )
	{
		index = 0;
		word[ index++ ] = strtok( line, " :," );
		while( ( word[ index ] = strtok( NULL, " :," ) ) ) index++;

		if( !word[ 0 ] )
			continue;
		
		civ = civ_to_num( word[ 0 ] );
		if( civ != -1 )
		{
			curciv = civ;
			fprintf( output, "\n\t--- %-10s", Civ[ curciv ].name );
			if( index == 2 )
			{
				count = atoi( word[ 1 ] );
				if( count < 0 )
					fprintf( output, " : %d units lost", -count );
				else
					fprintf( output, " : %d units gained", count );
			}
			fprintf( output, "\n" );
		}

		/* comment line */
		else if( !strncmp( word[ 0 ], "-", 1 ) )
		{
			/* do nothing */
		}

		/* eliminate city */
		else if( !strcmp( word[ 0 ], "ec" ) )
		{
			do_eliminatecity( curciv, index, word, output );
		}

		/* add city */
		else if( !strcmp( word[ 0 ], "ac" ) )
		{
			do_addcity( curciv, index, word, output );
		}

		/* reduce city */
		else if( !strcmp( word[ 0 ], "rc" ) )
		{
			do_reducecity( curciv, index, word, output );
		}

		/* add token */
		else if( !strcmp( word[ 0 ], "at" ) )
		{
			do_addtoken( curciv, index, word, output );
		}

		/* eliminate token */
		else if( !strcmp( word[ 0 ], "et" ) )
		{
			do_eliminatetoken( curciv, index, word, output );
		}

		else if( !strcmp( word[ 0 ], "calamity" ) )
		{
			int num = 0;
			ncal++;
			get_standings();
			fprintf( output, "\nStandings:" );
			for( civ = 0; civ < NumCivs; civ++ )
				if( Civ[ civ ].active == 1 )
				{
					fprintf( output, " %d.%s:%d", num + 1,
						Civ[ Standings[ num ] ].abbr,
						Civ[ Standings[ num ] ].score );
					num++;
				}

			civ = civ_to_num( word[ 2 ] );
			card = card_to_num( word[ 1 ] );

			fprintf( output, "\n" );
			/*
			fprintf( output, "\n%d. %s (%s)\n",
				ncal, Card[ card_to_num( word[ 1 ] ) ].name,
				Civ[ civ_to_num( word[ 2 ] ) ].name );
			*/
			fprintf( output, "\n%d. %s (%s)\n",
				ncal, Card[ card ].name, Civ[ civ ].name );
			CalamityOccured = 1;
			Civ[ civ ].discards[ card ] = 1;
		}
		else if( !strcmp( word[ 0 ], "unresolved" ) )
		{
			ncal++;
			fprintf( output, "\n%d. %s\n\n", ncal, "Unresolved Calamities" ); 
			for( count = 1; count < index; count += 2 )
			{
				card = card_to_num( word[ count ] );
				civ = civ_to_num( word[ count + 1 ] );
				fprintf( output, "\t%s (%s)\n",
					Card[ card ].name, Civ[ civ ].name );
				Civ[ civ ].discards[ card ] = 1;
			}
		}
		else if( !strcmp( word[ 0 ], "conversion" ) )
		{
			ncal++;
			fprintf( output, "\n%d. %s\n", ncal, "Monotheastic Conversions" ); 
		}
		else
		{
			fprintf( returnfile, ">>> syntax error\n" );
			zeus_exit( "command error" );
		}
	}
}

do_addplayer( line, word, output )
char line[ 128 ];
char *word[ 65 ];
FILE *output;
{
	int civ, area;

	fprintf( output, "%s", line );

	civ = civ_to_num( word[ 1 ] );
	if( civ == -1 )
	{
		fprintf( returnfile, ">>> bad civ name %s\n", word[ 1 ] );
		zeus_exit( "command error" );
	}

	/* players can't select pirates or barbarians */
	if( !strcmp( Civ[ civ ].abbr, "pir" ) )
	{
		fprintf( returnfile, ">>> can't play %s\n", word[ 1 ] );
		zeus_exit( "command error" );
	}
	if( !strcmp( Civ[ civ ].abbr, "bar" ) )
	{
		fprintf( returnfile, ">>> can't play %s\n", word[ 1 ] );
		zeus_exit( "command error" );
	}

	/* test for western expansion an iberia/italy start areas */
	area = area_to_num( "Baetica" );
	if( Map[ area ].active )
	{
		if( !strcmp( Civ[ civ ].abbr, "ita" ) )
		{
			fprintf( returnfile,
				">>> can't play %s with western expansion\n",
				word[ 1 ] );
			zeus_exit( "command error" );
		}
	}
	else
	{
		if( !strcmp( Civ[ civ ].abbr, "ibe" ) )
		{
			fprintf( returnfile,
				">>> can't play %s without western expansion\n",
				word[ 1 ] );
			zeus_exit( "command error" );
		}
	}

	Civ[ civ ].active = INPLAY;
	free( Civ[ civ ].player );
	Civ[ civ ].player = (char *)malloc( strlen( word[ 3 ] ) + 1 );
	strcpy( Civ[ civ ].player, word[ 3 ] );
	NumPlayers++;
	area = area_to_num( word[ 2 ] );
	if( area == -1 )
	{
		zeus_exit( "command error" );
	}
	if( !Map[ area ].active )
	{
		fprintf( returnfile, ">>> inactive area %s\n", word[ 2 ] );
		zeus_exit( "command error" );
	}
	if( Map[ area ].start != ( civ + 1 ) )
	{
		fprintf( returnfile, ">>> illegal start area %s\n", word[ 2 ] );
		zeus_exit( "command error" );
	}
	Map[ area ].tokens[ civ ] = 2;

	fprintf( output, " <done>\n" );
}

do_buildcity( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int civ, area;
	int resources, payment;

	area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't build city\n" );
		zeus_exit( "command error" );
	}

	if( Map[ area ].site )
		resources = 6;
	else
		resources = 12;

	if( index == 3 )
		payment = resources - Map[ area ].tokens[ curciv ];
	else
		payment = 0;

	if( !Map[ area ].city && ( Civ[ curciv ].cities <= MaxCities ) &&
		( ( Map[ area ].tokens[ curciv ] >= resources ) ||
			( Civ[ curciv ].arc && ( payment != 0 ) &&
			( payment <= Civ[ curciv ].treasury ) &&
				( Map[ area ].tokens[ curciv ] >= resources / 2 ) ) ))
	{
		if( payment > 0 )
		{
			Civ[ curciv ].treasury -= payment;
			Civ[ curciv ].stock += payment;
		}

		Civ[ curciv ].stock += Map[ area ].tokens[ curciv ];

		if( payment > 0 )
			fprintf( output, "%-20s %-20s %2d tokens %2d treasury\n",
				Map[ area ].name,Civ[ curciv ].name,
				Map[ area ].tokens[ curciv ],payment );
		else
			fprintf( output, "%-20s %-20s %2d tokens\n",
				Map[ area ].name,Civ[ curciv ].name,
				Map[ area ].tokens[ curciv ] );

		Map[ area ].tokens[ curciv ] = 0;
		Map[ area ].city = curciv + 1;
		Civ[ curciv ].cities++;
		CitiesBuilt = 1;
	}
}

do_reducecity( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't reduce city\n" );
		zeus_exit( "command error" );
	}
	Map[ area ].tokens[ curciv ] = Map[ area ].pop + Civ[ curciv ].agr;
	Map[ area ].city = 0;
	Civ[ curciv ].cities--;
	fprintf( output, "\t%-15s --> %d tokens\n",
		Map[ area ].name, Map[ area ].tokens[ curciv ] );
}

do_buildship( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int civ, area;
	int tokens, talents, port;

	area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't build ship\n" );
		zeus_exit( "command error" );
	}
	if( area_has_port( area ) )
	{
		fprintf( returnfile,
			">>> can't build ship in an area that has ports\n" );
		zeus_exit( "command error" );
	}
	if( index == 3 )
	{
		tokens = atoi( word[ 2 ] );
		talents = 2 - tokens;
	}
	else
	{
		tokens  = 0;
		talents = 2 - tokens;
	}

	port = area;
	if( Map[ area ].port )
		area = Map[ area ].port;

	if( ( Map[ area ].tokens[ curciv ] == 0 ) &&
		( Map[ area ].city != ( curciv + 1 ) ) )
	{
		fprintf( returnfile, ">>> no units in %s\n", word[ 1 ] );
		zeus_exit( "command error" );
	}
	else if( Map[ area ].tokens[ curciv ] < tokens )
	{
		fprintf( returnfile,
			">>> not enough tokens to build ship in %s\n",
				word[ 1 ] );
		zeus_exit( "command error" );
	}
	else if( Civ[ curciv ].treasury < talents )
	{
		fprintf( returnfile,
			">>> not enough treasury to build ship in %s\n",
				word[ 1 ]);
		zeus_exit( "command error" );
	}
	else
	{
		Map[ area ].tokens[ curciv ] -= tokens;
		Civ[ curciv ].treasury -= talents;
		Map[ port ].ships[ curciv ] += 1;
		Civ[ curciv ].ships++;
	}
}

do_maintainship( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int civ, area;
	int tokens, talents, port;

	area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't maintain ship\n" );
		zeus_exit( "command error" );
	}
	if( index == 3 )
	{
		tokens = atoi( word[ 2 ] );
		talents = 1 - tokens;
	}
	else
	{
		tokens  = 0;
		talents = 1 - tokens;
	}

	port = area;
	if( Map[ area ].port )
		area = Map[ area ].port;

	if( ( Map[ area ].oships[ curciv ] == 0 ) )
	{
		fprintf( returnfile, ">>> no ship in %s\n",
			word[ 1 ] );
		zeus_exit( "command error" );
	}
	else if( Map[ area ].tokens[ curciv ] < tokens )
	{
		fprintf( returnfile,
			">>> not enough tokens to maintain ship in %s\n",
				word[ 1 ] );
		zeus_exit( "command error" );
	}
	else if( Civ[ curciv ].treasury < talents )
	{
		fprintf( returnfile,
			">>> not enough treasury to maintain ship in %s\n",
				word[ 1 ] );
		zeus_exit( "command error" );
	}
	else
	{
		Map[ area ].tokens[ curciv ] -= tokens;
		Civ[ curciv ].treasury -= talents;
		Map[ port ].ships[ curciv ] += 1;
		Map[ port ].oships[ curciv ] -= 1;
		Civ[ curciv ].ships++;
	}
}

do_growall( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int area;

	for( area = 0; area < NumAreas; area++ )
	{
		if( Map[ area ].tokens[ curciv ] == 1 )
			Map[ area ].tokens[ curciv ] += 1;
		else if( Map[ area ].tokens[ curciv ] > 1 )
			Map[ area ].tokens[ curciv ] += 2;
	}
	Civ[ curciv ].census = num_tokens( curciv );
	Civ[ curciv ].stock = MaxTokens - Civ[ curciv ].census -
		Civ[ curciv ].treasury;
}

do_growtoken( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int area;
	int ntokens;

	area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't grow token\n" );
		zeus_exit( "command error" );
	}

	if( index == 3 )
		ntokens = atoi( word[ 2 ] );
	else
		ntokens = 1;

	if( ( Map[ area ].tokens[ curciv ] == 1 ) && ( ntokens == 1 ) )
		Map[ area ].tokens[ curciv ] += ntokens;
	else if( Map[ area ].tokens[ curciv ] > 1 )
		Map[ area ].tokens[ curciv ] += ntokens;
	else
	{
		fprintf( returnfile,
			">>> illegal growth in %s\n",
				Map[ area ].name );
		zeus_exit( "command error" );
	}
	Civ[ curciv ].census += ntokens;
	Civ[ curciv ].stock -= ntokens;
}

do_taxrate( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int income;
	Civ[ curciv ].taxrate = atoi( word[ 1 ] );
	income = Civ[ curciv ].cities * Civ[ curciv ].taxrate;
	Civ[ curciv ].treasury += income;
}

do_pillage( curciv, index, word, output )
int curciv, index;
char *word[ 65 ];
FILE *output;
{
	int area;
	int ntokens;

	area = area_to_num( word[ 1 ] );
	if( area == -1 )
	{
		fprintf( returnfile, ">>> can't pillage city\n" );
		zeus_exit( "command error" );
	}
	if( index == 3 )
		ntokens = atoi( word[ 2 ] );
	else
		ntokens = 3;
	if( ntokens <= Civ[ curciv ].stock )
	{
		Civ[ curciv ].treasury += ntokens;
		Civ[ curciv ].stock -= ntokens;
		if( output )
		{
			fprintf( output, "\n---%s\n",Map[ area ].name );
			fprintf( output, "%s pillages for %d treasury\n",
				Civ[ curciv ].name, ntokens );
		}
	}
}

modify_mapboard( input, output )
FILE *input;
FILE *output;
{
	char line[ 128 ], sline[ 128 ];
	char *word[ 65 ];
	int index, civ, curciv;
	int ordersgiven;

	curciv = -1;
	ordersgiven = 0;
	while( fgetline( input, line, 128 ) != -1 )
	{
		strcpy( sline, line );

		index = 0;
		word[ index++ ] = strtok( line, " :," );
		while( ( word[ index ] = strtok( NULL, " :," ) ) ) index++;

		if( !word[ 0 ] )
			continue;
		
		civ = civ_to_num( word[ 0 ] );
		if( civ != -1 )
		{
			if( !mail )
			{
				if( curciv != -1 )
				{
					if( ordersgiven )
						printf( "done\n" );
					else
						printf( "\n" );
				}
				printf( "%s...", Civ[ civ ].name );
			}
			ordersgiven = 0;
			curciv = civ;
		}

		/* comment line */
		else if( !strncmp( word[ 0 ], "-", 1 ) )
		{
			/* do nothing */
		}

		/* add a player to game */
		/* SYNTAX: addplayer <civilization> <start_area> <player_name> */
		else if( !strcmp( word[ 0 ], "addplayer" ) && ( index == 4 ) )
		{
			do_addplayer( sline, word, output );
		}

		/* build city */
		else if( !strcmp( word[ 0 ], "bc" ) )
		{
			do_buildcity( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* reduce city */
		else if( !strcmp( word[ 0 ], "rc" ) )
		{
			do_reducecity( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* build ship */
		else if( !strcmp( word[ 0 ], "bs" ) )
		{
			do_buildship( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* maintain ship */
		else if( !strcmp( word[ 0 ], "ms" ) )
		{
			do_maintainship( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* buy gold */
		else if( !strcmp( word[ 0 ], "buygold" ) )
		{
			Civ[ curciv ].buygold  = 1;
			ordersgiven = 1;
		}

		/* tax rate */
		else if( !strcmp( word[ 0 ], "tax" ) )
		{
			do_taxrate( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* pillage */
		else if( !strcmp( word[ 0 ], "pil" ) )
		{
			do_pillage( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* grow all tokens */
		else if( !strcmp( word[ 0 ], "ga" ) )
		{
			do_growall( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* grow token */
		else if( !strcmp( word[ 0 ], "gt" ) )
		{
			do_growtoken( curciv, index, word, output );
			ordersgiven = 1;
		}

		/* pass move, no orders */
		else if( !strcmp( word[ 0 ], "pass" ) )
		{
			ordersgiven = 1;
		}
		else if( !strcmp( word[ 0 ], "none" ) )
		{
			ordersgiven = 1;
		}

		/* Syntax Error */
		else
		{
			fprintf( returnfile, ">>> syntax error\n" );
			zeus_exit( "command error" );
		}
	}

	if( !mail && ( curciv != -1 ) )
	{
		if( ordersgiven )
			printf( "done\n" );
		else
			printf( "\n" );
	}
}

handle_move()
{
	FILE *fileptr;

	handle_ships();

	if( !mail )
		printf( "\nexecuting move orders...\n\n" );
	get_moves( NULL, NULL );

	if( !( fileptr = fopen( MAPBOARD1, "w+" ) ) )
		fopen_error( MAPBOARD1 );

	print_mapboard( fileptr, 1, 0 );
	fclose( fileptr );
}

handle_conflict()
{
	FILE *fileptr;
	char filename[ 128 ];
	char command[ 128 ];

	handle_move();

	if( !mail )
		printf( "\nresolving conflict...\n\n" );
	do_conflicts();

	if( !( fileptr = fopen( MAPBOARD2, "w+" ) ) )
		fopen_error( MAPBOARD2 );

	print_mapboard( fileptr, 2, 0 );
	fclose( fileptr );

	sprintf( filename, "%d/%02d01", Turn, Turn );
	fileptr = fopen( filename, "w+" );
	fprintf( fileptr, "Turn %2d - Movement/Conflict Summary\n\n", Turn );
	fclose( fileptr );

	sprintf( command, "cat %s >> %d/%02d01", MOVESFILE, Turn, Turn );
	system( command );

	fileptr = fopen( filename, "a" );
	fprintf( fileptr, "^L\n" );
	fclose( fileptr );

	sprintf( command, "cat %s >> %d/%02d01", CONFLICTSFILE, Turn, Turn );
	system( command );

	fileptr = fopen( filename, "a" );
	fprintf( fileptr, "^L\n" );
	print_mapboard( fileptr, 2, 0 );
	fclose( fileptr );
}

handle_cities()
{
	FILE *input, *output, *fileptr;
	char command[ 128 ];

	handle_conflict();

	CitiesBuilt = 0;

	if( !( input = fopen( NEWCITIESFILE, "r" ) ) )
		fopen_error( NEWCITIESFILE );

	if( !( output = fopen( CITIESFILE, "w+" ) ) )
		fopen_error( CITIESFILE );

	fprintf( output, "New Cities\n==========\n" );

	if( !mail )
		printf( "\nexecuting city orders...\n\n" );
	modify_mapboard( input, output );

	if( !CitiesBuilt )
		fprintf( output, "None\n" );

	fclose( input );
	fclose( output );

	reduce_surplus_population();

	if( !( fileptr = fopen( MAPBOARD3, "w+" ) ) )
		fopen_error( MAPBOARD3 );

	print_mapboard( fileptr, 3, 0 );
	fclose( fileptr );

	if( !mail )
	{
		printf( "\nreading city lists...\n\n" );
		get_city_lists();
	}

	check_for_city_support();
}

handle_pillage( output )
FILE *output;
{
	FILE *fileptr;

	if( !( fileptr = fopen( PILLAGEFILE, "r" ) ) )
		fopen_error( PILLAGEFILE );

	modify_mapboard( fileptr, output );
	fclose( fileptr );
}

list_areas( area_info)
int area_info[ MAX_AREAS ];
{
	int area, civ;

	tempfile = fopen( TEMPFILE, "w+" );
	for( area = 0; area < NumAreas; area++ )
	{
		if( !Map[ area ].active )
			continue;
		if( Map[ area ].geo == PORT )
			continue;
		if( Map[ area ].geo == SEA )
			continue;
		if( area_info[ area ] )
		{
			if( Map[ area ].city )
			{
				fprintf( tempfile, "%-22sC-%s ",
					Map[ area ].name, Civ[ Map[ area ].city - 1 ].abbr );
				for( civ = 0; civ < NumCivs; civ++ )
				{
					if( Map[ area ].tokens[ civ ] )
						fprintf( tempfile, "%d-%s ",
							Map[ area ].tokens[ civ ], Civ[ civ ].abbr );
				}
				fprintf( tempfile, "\n" );
			}
			else
			{
				fprintf( tempfile, "%-22s", Map[ area ].name );
				for( civ = 0; civ < NumCivs; civ++ )
				{
					if( Map[ area ].tokens[ civ ] )
						fprintf( tempfile, "%d-%s ",
							Map[ area ].tokens[ civ ], Civ[ civ ].abbr );
				}
				fprintf( tempfile, "\n" );
			}
		}
	}
	fclose( tempfile );
	send_file( TEMPFILE, "area list", sender );
}

find_areas( civ1, area_info, distance )
int civ1;
int area_info[ MAX_AREAS ];
int distance;
{
	int area1, area2, civ2;

	/*
	printf( "distance = %d\n\n", distance );
	*/
	for( area1 = 0; area1 < NumAreas; area1++ )
	{
		if( distance == 0 )
		{
			if( units_in_area( civ1, area1 ) )
			{
				for( civ2 = 0; civ2 < NumCivs; civ2++ )
				{
					if( civ1 == civ2 )
						continue;
					if( Map[ area1 ].tokens[ civ2 ] )
						area_info[ area1 ] = 1;
				}
			}
		}
		else
		{
			for( area2 = 0; area2 < Map[ area1 ].nland; area2++ )
			{
				for( civ2 = 0; civ2 < NumCivs; civ2++ )
				{
					if( civ1 == civ2 )
						continue;
					if( Map[ *(Map[ area1 ].land + area2) ].tokens[ civ2 ] )
					{
						area_info[ *(Map[ area1 ].land + area2) ] = 1;
						find_areas( civ1, area_info, distance - 1);
					}
				}
			}
			for( area2 = 0; area2 < Map[ area1 ].nsea; area2++ )
			{
				for( civ2 = 0; civ2 < NumCivs; civ2++ )
				{
					if( civ1 == civ2 )
						continue;
					if( Map[ *(Map[ area1 ].sea + area2) ].tokens[ civ2 ] )
					{
						area_info[ *(Map[ area1 ].sea + area2) ] = 1;
						find_areas( civ1, area_info, distance - 1);
					}	
				}
			}
		}
	}
}

handle_civships( input, output )
FILE *input;
FILE *output;
{
	FILE *fileptr;

	if( !( fileptr = fopen( SHIPORDER( Civ[ logged_in ].abbr ), "w+" ) ) )
		fopen_error( SHIPORDER( Civ[ logged_in ].abbr ) );

	modify_mapboard( input, output );

	fclose( fileptr );
}

handle_ships()
{
	FILE *fileptr;
	char filename[ 128 ], command[ 128 ];

	get_mapboard();

	if( !( fileptr = fopen( SHIPSFILE, "r" ) ) )
		fopen_error( SHIPSFILE );

	if( !mail )
		printf( "\nexecuting ship orders...\n\n" );
	modify_mapboard( fileptr, NULL );
	fclose( fileptr );

	fileptr = fopen( MAPBOARD0, "w+" );
	print_mapboard( fileptr, 0, 0 );
	fclose( fileptr );

	sprintf( filename, "%d/%02d00", Turn, Turn );
	fileptr = fopen( filename, "w+" );
	fprintf( fileptr, "Turn %2d - Ship Summary\n\n", Turn );
	fclose( fileptr );

	sprintf( command, "cat %s >> %d/%02d00", SHIPSFILE, Turn, Turn );
	system( command );

	fileptr = fopen( filename, "a" );
	fprintf( fileptr, "\n" );
	print_mapboard( fileptr, 0, 0 );
	fclose( fileptr );
}

handle_growth()
{
	FILE *fileptr, *output;
	char filename[ 128 ];
	int deck, card;

	handle_calamities();

	if( !( fileptr = fopen( PURCHASESFILE, "r" ) ) )
		fopen_error( PURCHASESFILE );
	if( !mail )
		printf( "\nexecuting purchase orders...\n\n" );
	handle_purchases( fileptr, NULL );
	fclose( fileptr );

	shuffle_discards();

	update_ast_and_techs();

	if( !( fileptr = fopen( GROWTHFILE, "r" ) ) )
		fopen_error( GROWTHFILE );
	if( !mail )
		printf( "\nexecuting growth orders...\n\n" );
	modify_mapboard( fileptr, NULL );
	fclose( fileptr );

	if( !( fileptr = fopen( MAPBOARD6, "w+" ) ) )
		fopen_error( MAPBOARD6 );
	print_mapboard( fileptr, 6, 0 );
	fclose( fileptr );

	print_final_summary();
}

handle_map()
{
	FILE *fileptr;

	mail = 1;

	if( !( fileptr = fopen( MAPBOARDINFO, "w+" ) ) )
		fopen_error( MAPBOARDINFO );

	get_latest_mapboard();

	print_mapboard( fileptr, Phase - 1, 1 );

	fclose( fileptr );

	mail = 0;
}
