
#include "zeus.h"

int	total_civcard_points( civ )
int civ;
{
	int tech;
	int points;

	points = 0;
	for( tech = 0; tech < NUM_TECHS; tech++ )
		if( Civ[ civ ].tech[ tech ] )
			points += Tech[ tech ].value;
	return( points );
}

int tech_to_num( name )
char *name;
{
	int tech;
	for( tech = 0; tech < NUM_TECHS; tech++ )
		if( !strcmp( Tech[ tech ].abbr, name ) )
			return( tech );
	for( tech = 0; tech < NUM_TECHS; tech++ )
		if( !strcmp( Tech[ tech ].name, name ) )
			return( tech );
	return( -1 );
}

get_civcards()
{
	FILE *fileptr;
	char line[ 128 ];
	char *word[ 65 ];
	int index, tech;

	if( !( fileptr = fopen( CIVCARDSFILE, "r" ) ) )
		fopen_error( CIVCARDSFILE );

	tech = 0;
	while( fgetline( fileptr, line, 128 ) != -1 )
	{
		index = 0;
		word[ index++ ] = strtok( line, " -:," );
		while( ( word[ index ] = strtok( NULL, " -:," ) ) ) index++;

		strcpy( Tech[ tech ].name, word[ 0 ] );
		strcpy( Tech[ tech ].abbr, word[ 1 ] );
		Tech[ tech ].value = atoi( word[ 2 ] );

		Tech[ tech ].type = civcardtype_to_num( word[ 3 ] );

		if( Tech[ tech ].type == OTHER )
		{
			Tech[ tech ].subtype1 = civcardtype_to_num( word[ 4 ] );
			Tech[ tech ].subtype2 = civcardtype_to_num( word[ 5 ] );
			if( index == 7 )
				Tech[ tech ].prereq = tech_to_num( word[ 6 ] );
			else
				Tech[ tech ].prereq = -1;
		}
		else
		{
			Tech[ tech ].subtype1 = Tech[ tech ].type;
			Tech[ tech ].subtype2 = Tech[ tech ].type;
			if( index == 5 )
				Tech[ tech ].prereq = tech_to_num( word[ 4 ] );
			else
				Tech[ tech ].prereq = -1;
		}

		tech++;
	}

	fclose( fileptr );
}

get_credits()
{
	FILE *fileptr;
	char line[ 128 ];
	char *word[ 65 ];
	int index, tech, nline, ntech;

	if( !( fileptr = fopen( CREDITSFILE, "r" ) ) )
		fopen_error( CREDITSFILE );

	nline = 0;
	tech = 0;
	while( fgetline( fileptr, line, 128 ) != -1 )
	{
		nline++;

		index = 0;
		word[ index++ ] = strtok( line, " -:," );
		while( ( word[ index ] = strtok( NULL, " :," ) ) ) index++;

		if( nline == 1 )
			continue;

		for( ntech = 0; ntech < NUM_TECHS; ntech++ )
			Credits[ tech ][ ntech ] = atoi( word[ ntech + 1 ] );

		tech++;
	}

	fclose( fileptr );
}

print_civcards( civ, output )
int civ;
FILE *output;
{
	int tech, type, otype;

	otype = -1;
	fprintf(output,"\n--- %s ( %d )",
		Civ[ civ ].name, total_civcard_points( civ ) );
	for( type = 0; type < 6; type++ )
	{
		for( tech = 0; tech < NUM_TECHS; tech++ )
		{
			if( Tech[ tech ].type == type )
			{
				if( type != otype )
				{
					fprintf( output, "\n%-11s", CivcardClass[ type ] );
					otype = type;
				}
				if( Civ[ civ ].tech[ tech ] )
					fprintf( output, "%-3s ", Tech[ tech ].abbr );
				else
					fprintf( output, "--- " );
			}
		}
	}
	fprintf( output, "\n" );
}

list_civcards( filename, mode )
char *filename;
int mode;
{
	FILE *output;
	int civ, tech, index;
	int credits, cost;
	int type, otype;
	int newtype = 0;

	if( mode )
		output = fopen( filename, "w+" );
	else
		output = fopen( filename, "a" );
	if( !output )
		fopen_error( filename );

	fprintf( output, "\n--- Card Manifest" );
	otype = -1;
	for( type = 0; type < 6; type++ )
	{
		for( tech = 0; tech < NUM_TECHS; tech++ )
		{
			if( Tech[ tech ].type == type )
			{
				if( type != otype )
				{
					fprintf( output, "\n%-11s", CivcardClass[ type ] );
					otype = type;
				}
				fprintf( output, "%-3s:%3d ",
					Tech[ tech ].abbr, Tech[ tech ].value );
			}
		}
	}
	fprintf( output, "\n" );
	fprintf( output, "           " );
	for( tech = 0; tech < NUM_TECHS; tech++ )
	{
		if( Tech[ tech ].type == OTHER )
		{
			fprintf( output, "%3s:(%s/%s) ",
				Tech[ tech ].abbr,
				CivcardClassAbbr[ Tech[ tech ].subtype1 ],
				CivcardClassAbbr[ Tech[ tech ].subtype2 ] );
		}
	}
	fprintf( output, "\n" );

	for( civ = 0; civ < NumCivs; civ++ )
	{
		if( !Civ[ civ ].active )
			continue;
		print_civcards( civ, output );
		otype = -1;
		for( type = 0; type < 6; type++ )
		{
			for( tech = 0; tech < NUM_TECHS; tech++ )
			{
				if( Tech[ tech ].type == type )
				{
					if( type != otype )
						newtype = 1;

					if( !Civ[ civ ].tech[ tech ] &&
						( ( ( Tech[ tech ].prereq >= 0 ) &&
						Civ[ civ ].tech[ Tech[ tech ].prereq ] ) ||
						( Tech[ tech ].prereq < 0 ) ) )
					{
						if( newtype )
						{
							fprintf( output,
								"\n%-11s", CivcardClass[ type ] );
							otype = type;
							newtype = 0;
						}
						credits = 0;
						for( index = 0; index < NUM_TECHS; index++ )
					 	{
							if( Civ[ civ ].tech[ index ] &&
								( Credits[ index ][ tech ] > 0 ) )
							{
								credits += Credits[ index ][ tech ];
							}
						}
						cost = Tech[ tech ].value - credits;
						fprintf(output,"%-3s:%3d(%3d) ",
							Tech[ tech ].abbr, cost, credits);
					}
				}
			}
		}
		fprintf( output, "\n" );
	}

	fclose( output );

	if( mode )
		send_file( filename, "civcards", sender );
}

set_tech_flag( string, civ )
char *string;
int civ;
{

	if( !strcmp( "pot", string ) )
		Civ[ civ ].pot = 1;
	else if( !strcmp( "clo", string ) )
		Civ[ civ ].clo = 1;
	else if( !strcmp( "met", string ) )
		Civ[ civ ].met = 1;
	else if( !strcmp( "agr", string ) )
		Civ[ civ ].agr = 1;
	else if( !strcmp( "roa", string ) )
		Civ[ civ ].roa = 1;
	else if( !strcmp( "min", string ) )
		Civ[ civ ].min = 1;
	else if( !strcmp( "ast", string ) )
		Civ[ civ ].ast = 1;
	else if( !strcmp( "coi", string ) )
		Civ[ civ ].coi = 1;
	else if( !strcmp( "med", string ) )
		Civ[ civ ].med = 1;
	else if( !strcmp( "dap", string ) )
		Civ[ civ ].dap = 1;
	else if( !strcmp( "mus", string ) )
		Civ[ civ ].mus = 1;
	else if( !strcmp( "arc", string ) )
		Civ[ civ ].arc = 1;
	else if( !strcmp( "dei", string ) )
		Civ[ civ ].dei = 1;
	else if( !strcmp( "enl", string ) )
		Civ[ civ ].enl = 1;
	else if( !strcmp( "mon", string ) )
		Civ[ civ ].mon = 1;
	else if( !strcmp( "the", string ) )
		Civ[ civ ].the = 1;
	else if( !strcmp( "law", string ) )
		Civ[ civ ].law = 1;
	else if( !strcmp( "mil", string ) )
		Civ[ civ ].mil = 1;
	else if( !strcmp( "dem", string ) )
		Civ[ civ ].dem = 1;
	else if( !strcmp( "phi", string ) )
		Civ[ civ ].phi = 1;
	else if( !strcmp( "mys", string ) )
		Civ[ civ ].mys = 1;
	else if( !strcmp( "lit", string ) )
		Civ[ civ ].lit = 1;
	else if( !strcmp( "eng", string ) )
		Civ[ civ ].eng = 1;
	else if( !strcmp( "mat", string ) )
		Civ[ civ ].mat = 1;
}

get_techs()
{
	FILE *fileptr;
	char line[ 128 ];
	char *word[ 65 ];
	int nwrd, index, civ;

	if( !( fileptr = fopen( TECHSFILE, "r" ) ) )
		fopen_error( TECHSFILE );

	while( fgetline( fileptr, line, 128 ) != -1 )
	{
		index = 0;
		word[ index++ ] = strtok( line, " -:," );
		while( ( word[ index ] = strtok( NULL, " -:," ) ) ) index++;

		civ = civ_to_num( word[ 0 ] );

		for( nwrd = 1; nwrd < index; nwrd++ )
		{
			Civ[ civ ].tech[ tech_to_num( word[ nwrd ] ) ] = 1;
			set_tech_flag( word[ nwrd ], civ );
		}
	}

	fclose( fileptr );
}

handle_purchases( input, output )
FILE *input;
FILE *output;
{
	char line[ 128 ];
	char *word[ 65 ];
	int index, civ, curciv;
	int mine;
	short cardtype;
	short number;
	short count;
	int ordersgiven;

	while( fgetline( input, line, 128 ) != -1 )
	{
		index = 0;
		word[ index++ ] = strtok( line, " :,-" );
		while( ( word[ index ] = strtok( NULL, " :,-" ) ) ) index++;

		if( !word[ 0 ] )
			continue;
		
		civ = civ_to_num( word[ 0 ] );
		if( civ != -1 )
		{
			curciv = civ;
			Civ[ curciv ].money  = 0;
			Civ[ curciv ].change = 0;
			Civ[ curciv ].cardmined = -1;
			ordersgiven = 0;
			mine = -1;
		}
		else if( !strncmp( word[ 0 ], "-", 1 ) )
		{
		}
		/* discard cards */
		else if( !strcmp( word[ 0 ], "discard" ) )
		{
			ordersgiven = 1;
			for( count = 1; count < index; count += 2 )
			{
				number = atoi( word[ count ] );
				if( !strcmp( word[ count + 1 ], "T" ) )
				{
					if( number <= Civ[ curciv ].treasury )
					{
						Civ[ curciv ].change   += number;
						Civ[ curciv ].money    += number;
						Civ[ curciv ].stock    += number;
						Civ[ curciv ].treasury -= number;
					}
					else
					{
						fprintf( returnfile,
							">>> %s tried to spend %d from treasury\n",
							Civ[ curciv ].name, number );
						fprintf( returnfile,
							">>> only has %d\n",
							Civ[ curciv ].treasury );
						zeus_exit( "treasury error" );
					}
				}
				else
				{
					cardtype = card_to_num( word[ count + 1 ] );
					if( number <= Civ[ curciv ].cards[ cardtype ] )
					{
						Civ[ curciv ].discards[ cardtype ] += number;
						Civ[ curciv ].cards[ cardtype ]    -= number;
						Civ[ curciv ].ncards               -= number;
						if( ( mine == cardtype ) &&
							( number < Card[ cardtype ].number ) )
						{
							Civ[ curciv ].discard_value[ cardtype ] =
								( number + 1 ) * ( number + 1 ) *
								( Card[ cardtype ].deck + 1 );
							Civ[ curciv ].money +=
								Civ[ curciv ].discard_value[ cardtype ];
							Civ[ curciv ].cardmined = cardtype;
						}
						else
						{
							Civ[ curciv ].discard_value[ cardtype ] =
								number * number *
								( Card[ cardtype ].deck + 1 );
							Civ[ curciv ].money +=
								Civ[ curciv ].discard_value[ cardtype ];
						}
					}
					else
					{
						fprintf( returnfile,
							">>> %s tried to discard %d-%s\n",
							Civ[ curciv ].name, number, Card[ cardtype ].abbr );
						fprintf( returnfile,
							">>> only has %d-%s\n",
							Civ[ curciv ].cards[ cardtype ],
							Card[ cardtype ].abbr );
						zeus_exit( "discard error" );
					}
				}
			}
		}

		/* mine card */
		else if( !strcmp( word[ 0 ], "mine" ) )
		{
			ordersgiven = 1;
			cardtype = card_to_num( word[ 1 ] );
			if( Civ[ curciv ].min && Card[ cardtype ].mine )
				mine = cardtype;
		}

		/* buy cards */
		else if( !strcmp( word[ 0 ], "buy" ) )
		{
			ordersgiven = 1;
			Civ[ curciv ].moneyleft = Civ[ curciv ].money;
			for( count = 1; count < index; count++ )
			{
				int cardcost, technum;
				technum = tech_to_num( word[ count ] );
				if( ( Tech[ technum ].prereq >= 0 ) &&
					( Civ[ curciv ].tech[ Tech[ technum ].prereq ] == 0 ) )
				{
					fprintf( returnfile,
						">>> %s does not have pre-requisite for %s\n",
						Civ[ curciv ].name, Tech[ technum ].name );
					fprintf( returnfile, ">>> %s could not purchase %s\n",
						Civ[ curciv ].name, Tech[ technum ].name );
					zeus_exit( "purchase error" );
				}
				cardcost = Tech[ technum ].value;
				for( number = 0; number < NUM_TECHS; number++ )
			 	{
					if( Civ[ curciv ].tech[ number ] &&
						( Credits[ number ][ technum ] > 0 ) )
					{
						cardcost -= Credits[ number ][ technum ];
					}
				}
				if( cardcost <= Civ[ curciv ].moneyleft )
				{
					Civ[ curciv ].newtech[ technum ] = 1;
					Civ[ curciv ].techcost[ technum ] = cardcost;
					Civ[ curciv ].techcredits[ technum ] =
						Tech[ technum ].value -
						Civ[ curciv ].techcost[ technum ];
					Civ[ curciv ].moneyleft -= cardcost;
				}
				else
				{
					fprintf( returnfile,
						">>> %s does not have enough money to purchase %s\n",
						Civ[ curciv ].name, Tech[ technum ].name );
					fprintf( returnfile, ">>> %s could not purchase %s\n",
						Civ[ curciv ].name, Tech[ technum ].name );
					zeus_exit( "purchase error" );
				}
			}
			for( number = 0; number < NUM_TECHS; number++ )
			{
				if( Civ[ curciv ].newtech[ number ] )
				{
					Civ[ curciv ].tech[ number ] = 1;
					set_tech_flag( Tech[ number ].abbr, curciv );
				}
			}
		}
		else if( !strcmp( word[ 0 ], "pass" ) )
		{
		}
		else if( !strcmp( word[ 0 ], "none" ) )
		{
		}

		else
		{
			fprintf( returnfile, ">>> syntax error\n" );
			zeus_exit( "command error" );
		}
	}
}
