#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

/* Include file for zeus */
#ifdef sunos
#define RAND_MAX	0xFFFFFFFF
#endif

/* civ definitions */
#define INACTIVE       0  /* used for Pirates, Barbarians */
#define INPLAY         1  /* a standard player's civilization */
#define OUTPLAY        2  /* civ of a player who exits game early */

/* basic files independent of turns and phases in zeus home directory */
#define RETURNFILE     "returnfile"           /* created by zeus */
#define TEMPFILE       "tempfile"             /* created by zeus */
#define MAILLOGFILE    "maillog"              /* created by zeus */
#define TURNFILE       "TURN"                 /* created by moderator */
#define PHASEFILE      "PHASE"                /* created by moderator */
#define STARTFILE      "START"

/* static files created at the beginning of the game by moderator */
#define CIVCARDSFILE   "world/civcards"       /* civ card manifest */
#define TRADECARDSFILE "world/tradecards"     /* trade card manifest */
#define CREDITSFILE    "world/credits"        /* credits chart */
#define ASTTEMPLATE    "world/astchart"       /* ast chart */
#define CIVSTEMPLATE   "world/civs"           /* civs involved in game */
#define MAPFILE        "world/map"            /* map area definitions */
#define PATHSFILE      "world/paths"          /* map area paths */
#define HELPFILE       "world/help"           /* zeus help file */
#define CALFILE        "world/calamities"
#define YEARSFILE      "world/years"
#define RULESFILE      "world/rules"

#define PARAMSFILE     "world/PARAMS"         /* game specific parameters */
#define STARTTIMEFILE  "world/STARTTIME"      /* time game started in secs */
#define CIVSFILE       "world/CIVS"           /* civs involved in game */
#define PASSWORDSFILE  "world/PASSWORDS"      /* passwords of above civs */

/* files existing in separate turn directories */

/* summary files created by zeus */
#define CONFLICTSFILE  turn("conflicts")      /* conflicts summary */
#define CITIESFILE     turn("cities")         /* cities summary */
#define NEWCARDSFILE   turn("newcards")       /* number cards dealt to civs */
#define DEALFILE       turn("deal")           /* secret list of cards dealt */
#define NEWDECKSFILE   turn("decks.1")        /* decks after deal */
#define LASTDECKSFILE  turn("decks.2")        /* decks after discards */
#define PROPOSALSFILE  turn("proposals")      /* proposal file */
#define TRADESFILE     turn("trades")         /* list of trades */
#define SEEDFILE       turn("seed")           /* random number seed for turn */
#define LOOTFILE       turn("loot")           /* looting summary */
#define CALLOCFILE     turn("calloc")         /* calamity locations */
#define CALLOCNEWFILE  turn("calloc.2")       /* calamity locs after discard */

#define MAPBOARDFILE   turn("mapboard")       /* mapboard at start of turn */
#define MAPBOARD0      turn("mapboard.0")     /* map after ships */
#define MAPBOARD1      turn("mapboard.1")     /* map after movement */
#define MAPBOARD2      turn("mapboard.2")     /* map after conflict */
#define MAPBOARD3      turn("mapboard.3")     /* map after cities */
#define MAPBOARD4      turn("mapboard.4")     /* map after trade cards dealt */
#define MAPBOARD5      turn("mapboard.5")     /* map after calamities */
#define MAPBOARD6      turn("mapboard.6")     /* map after growth */
#define MAPBOARDINFO   turn("mapboard.info")  /* map info */

/* files created by moderator */
/* order files */
#define SHIPSFILE      turn("ord.ship")          /* ship orders */
#define MOVESFILE      turn("ord.move")          /* movement orders */
#define PILLAGEFILE    turn("ord.pill")          /* pillage/loot orders */
#define NEWCITIESFILE  turn("ord.city")          /* city building orders */
#define GOLDFILE       turn("ord.gold")          /* gold purchase orders */
#define CALAMITIESFILE turn("ord.cala")          /* calamity orders */
#define GROWTHFILE     turn("ord.grow")          /* growth/tax orders */
#define PURCHASESFILE  turn("ord.purc")          /* purchases/discards */
#define LISTSFILE      turn("ord.list")          /* city reduce/elim lists */

/* data files */
#define TECHSFILE      turn("techs")          /* techs at start of turn*/
#define TECHSNEWFILE   turn("techs.1")        /* techs at start of turn*/
#define ASTPOSFILE     turn("ast")            /* ast pos at start of turn */
#define ASTPOSNEWFILE  turn("ast.1")          /* ast pos at end of turn */
#define ASTFILE        turn("AST")            /* ast chart at start of turn */
#define ASTNEWFILE     turn("AST.1")          /* ast chart at start of turn */
#define DECKSFILE      turn("decks")          /* decks at start of turn */

/* card files */
#define CARDSFILE(civ) civturn("cards",civ)
#define SHIPORDER(civ) civturn("ships",civ)
#define CARDSNEWFILE(civ) civturn("CARDS",civ)

#define SEA     0
#define COAST   1
#define INLAND  2
#define PORT    3 /* used for areas with two coasts, like Delphi */

#define CRAFT          0
#define SCIENCE        1
#define ART            2
#define CIVIC          3
#define RELIGION       4
#define OTHER          5

#define MAX_CIVS       16   /* max number of civs in a game */
#define MAX_AREAS      256  /* max number of areas in a game */
#define SHIP_CAPACITY  5    /* max number of tokens ship can carry */
#define SHIP_MOVE      4    /* number of moves for a standard ship */
#define MAX_CARD_TYPES 32   /* max number of different trade cards in game */
#define NUM_DECKS      9    /* number of trade card decks */
#define MAX_CARDS      32   /* maximum number of cards per trade deck */
#define NUM_TECHS      24   /* number of civ advances */
#define NUM_PHASES     8    /* number of phases per turn */
#define AST_SIZE       16   /* number of squares on AST chart */
#define HAND_SIZE      8

#define MAX_CITIES     18   /* absolute max number of cities per civ */

#define MAX_FLOODS     8
#define MAX_VOLCS      8

typedef struct
{
	short civ1;  /* proposing civ */
	short civ2;  /* other civ */
	short nsend; /* number of cards civ1 is sending */
	short nget;  /* number of cards civ2 is sending */
	short vcard[ MAX_CARD_TYPES ]; /* visible cards */
	short ecard[ MAX_CARD_TYPES ]; /* expected cards */
	short acard[ MAX_CARD_TYPES ]; /* all cards */
	short scard[ MAX_CARD_TYPES ]; /* secret cards */
	short complete;
} _proposal;

/* trade card definitions */
typedef struct 
{
	char         *name;       /* name of card */
	char         *abbr;       /* abbrev for name */
	unsigned int cal:1;       /* 1 = calamity; 0 = commodity */
	unsigned int deck:4;      /* deck card is in; 1 to NUM_DECKS */
	unsigned int number:4;    /* number of this particular card */
	unsigned int tradeable:1; /* 1 = tradable; 0 = not tradable */
	unsigned int mine:1;
} _card;

/* civ card definitions */
typedef struct
{
	int  value;      /* base cost of card */
	int  type;       /* what type of card, e.g craft, ... */
	int  prereq;     /* a prereq card the card may have */
	char name[ 32 ]; /* name of card */
	char abbr[ 8 ];  /* abbrev for name */
	int subtype1;
	int subtype2;
} _tech;

typedef struct
{
	char         *player;                   /* name of player */
	char         *name;                     /* name of civ */
	char         *abbr;                     /* abbrev for civ name */
	char         password[ 9 ];             /* civ password */
	short        census;                    /* tokens on board */
	short        stock;                     /* tokens in stock */
	short        treasury;                  /* tokens in treasury */
	short        cities;                    /* number of cities on board */
	short        ships;                     /* number of ships on board */
	short        newcards[ 2 * NUM_DECKS ]; /* cards newly dealt */
	short        ndraws;                    /* number of cards newly dealt */
	short        ncards;                    /* number of cards in hand */
	short        cards[ MAX_CARD_TYPES ];   /* number of each type of card */
	short        tech[ NUM_TECHS ];         /* 1 = civ has card */
	short        AST[ 9 ];                  /* ast info */
	short        astpos;                    /* ast position */
	short        astnew;                    /* new ast position */
	short        city_reduce[ MAX_CITIES ];
	short        city_elim[ MAX_CITIES ];
	short        score;
	short        taxrate;

	short        discards[ MAX_CARD_TYPES ];
	short        discard_value[ MAX_CARD_TYPES ];
	short        change;
	short        cardmined;
	short        money;
	short        moneyleft;
	short        newtech[ NUM_TECHS ];
	short        techcredits[ NUM_TECHS ];
	short        techcost[ NUM_TECHS ];

	unsigned int moved:1;                   /* 1 = civ has moved */
	unsigned int active:2;                  /* 0 = inactive, 1 = active */
	unsigned int buygold:2;                 /* will buy gold */

	unsigned int pot:1; /* duplicated from tech above for easy access to data */
	unsigned int clo:1;
	unsigned int met:1;
	unsigned int agr:1;
	unsigned int roa:1;
	unsigned int min:1;
	unsigned int ast:1;
	unsigned int coi:1;
	unsigned int med:1;
	unsigned int dap:1;
	unsigned int mus:1;
	unsigned int arc:1;
	unsigned int dei:1;
	unsigned int enl:1;
	unsigned int mon:1;
	unsigned int the:1;
	unsigned int law:1;
	unsigned int mil:1;
	unsigned int dem:1;
	unsigned int phi:1;
	unsigned int mys:1;
	unsigned int lit:1;
	unsigned int eng:1;
	unsigned int mat:1;

	unsigned int    :0;
} _civ;


typedef struct {
	char         *name;       /* name of area */
	char         *abbr;       /* abbrev for area */
	unsigned int active  : 1; /* 1= active; 0 = inactive */
	unsigned int pop     : 3; /* population level */
	unsigned int site    : 2; /* 0=none, 1=no-flood, 2=flood */
	unsigned int flood   : 3; /* 0=no, >1=yes */ 
	unsigned int volcano : 2; /* 0=no, >1=yes */
	unsigned int geo     : 2; /* 0=sea, 1=coast, 2=inland */
	unsigned int start   : 4; /* starting civ */
	unsigned int         : 0;
	short        port;        /* area that this area serves as a port */
							  /* ports are used for multi coast areas */

	short        nland;       /* number of land paths */
	short        *land;       /* pointer to list of land paths */
	short        nsea;        /* number of sea paths */
	short        *sea;        /* pointer to list of sea paths */

	unsigned int city    : 4;        /* 0 = no city; >1 = city exists */
	unsigned int         : 0;
	short        tokens[ MAX_CIVS ];  /* number of tokens of that civ */
	short        ships [ MAX_CIVS ];  /* number of ships of that civ */
	short        oships [ MAX_CIVS ]; /* used for ship maintenace */

	short        dtok;                /* used for movement */
	short        dshp;                /* used for movement */
} _map;

#ifdef MAIN

#define EXTERN	/**/

_proposal Proposal[ 16 ];
_card     Card[ MAX_CARD_TYPES ];
_tech     Tech[ NUM_TECHS ];
_civ      Civ[ MAX_CIVS ];
_map      Map[ MAX_AREAS ];

/* INITIALIZED DATA */
/* number of civs in the game including pirates and barbarians */
int NumCivs = 0;

/* number of cities per civ */
int MaxCities = 9;

/* number of tokens per civ */
int MaxTokens = 55;

/* number of human players in the game */
int NumPlayers = 0;

/* number of active areas in the game */
int NumAreas = 0;

/* number of current trade proposals */
short NumProps = 0;

/* number of different trade card types in game like gold, treachery, etc... */
int NTCards = 0;

/* turn of the game */
int Turn = -1;

/* phase of the game; index into PhaseName */
char *PhaseName[ NUM_PHASES ] =
{
	"Start",      /* 0 */
	"Ships",      /* 1 */
	"Movement",   /* 2 */
	"Conflict",   /* 3 */
	"Cities",     /* 4 */
	"Trade",      /* 5 */
	"Calamities", /* 6 */
	"Growth",     /* 7 */
};

char *PhaseStatus[ 2 ] = { "Active", "Complete" };

char *YesNo[ 2 ] = { "No", "Yes" };

/* number of cities required for advancement */
int CitiesRequired[ 5 ] = { 0, 2, 3, 4, 5 };

char *Ages[ 6 ] =
{
	"Pre-History",
	"Stone",
	"Early Bronze",
	"Late Bronze",
	"Early Iron",
	"Late Iron"
};

char *CivcardClass[ 6 ] =
{
	"Crafts",
	"Sciences",
	"Arts",
	"Civics",
	"Religions",
	"Other"
};

char *CivcardClassAbbr[ 5 ] =
{
	"cra", "sci", "art", "civ", "rel"
};

int CalamityOccured = 0;
int ConflictOccured = 0;
int CitiesBuilt = 0;
int SurplusPopulation = 0;
int ShowCalamities = 0;

#else /* MAIN */

#define EXTERN	extern /**/

/* INITIALIZED DATA */
EXTERN int MaxCities;
EXTERN int MaxTokens;
EXTERN int NumCivs;
EXTERN int NumPlayers;
EXTERN int NumAreas;
EXTERN short NumProps;
EXTERN int NTCards;
EXTERN int Turn;
EXTERN int CitiesRequired[ 5 ];
EXTERN char *PhaseName[ NUM_PHASES ];
EXTERN char *PhaseStatus[ 2 ];
EXTERN char *Ages[ 6 ];
EXTERN char *CivcardClass[ 6 ];
EXTERN char *CivcardClassAbbr[ 5 ];
EXTERN char *YesNo[ 2 ];
#endif /* !MAIN */

/* UNINITIALIZED DATA */
EXTERN int NewGame;
EXTERN short mail;
EXTERN short logged_in;

EXTERN short CensusOrder[ MAX_CIVS ];
EXTERN short Standings[ MAX_CIVS ];

/* unimportant seed stuff */
EXTERN unsigned int Seed;

/* civ card credits chart */
EXTERN short Credits[ NUM_TECHS ][ NUM_TECHS ];

/* ordered list of each deck's constituents */
EXTERN short Deck[ NUM_DECKS ][ MAX_CARDS ];
/* number of cards currently in each deck */
EXTERN short NumCards[ NUM_DECKS ];
/* maximum number of cards in each deck */
EXTERN short MaxCards[ NUM_DECKS ];

/* unimportant start time of game */
EXTERN time_t starttime;
EXTERN int day;

/* various files */
EXTERN FILE *tempfile;
EXTERN FILE *returnfile;
EXTERN FILE *maillog;

/* the sender of the mail message; e-mail address */
EXTERN char sender[ 128 ];

EXTERN int Years[ 20 ];
EXTERN int Phase;
EXTERN int PhaseComplete;

EXTERN int CalamityOccured;
EXTERN int ConflictOccured;
EXTERN int CitiesBuilt;
EXTERN int SurplusPopulation;
EXTERN int ShowCalamities;

EXTERN char *turn();
EXTERN char *civturn();

EXTERN _proposal Proposal[ 16 ];
EXTERN _card     Card[ MAX_CARD_TYPES ];
EXTERN _tech     Tech[ NUM_TECHS ];
EXTERN _civ      Civ[ MAX_CIVS ];
EXTERN _map      Map[ MAX_AREAS ];

EXTERN int CalamityLocation[ 12 ][ 2 ];
EXTERN short OldNumCards[ NUM_DECKS ];
